@@ -39,7 +39,7 @@
 #include <string.h>
 #include <sys/types.h>
 #include <dirent.h>
-#if !defined(N_PLAT_NLM) && defined(linux)
+#if defined(linux)
 /*
  * These headers are needed to get __USE_LARGEFILE, __USE_LARGEFILE64,
  * and SYS__llseek.
@@ -69,7 +69,6 @@
 #include <sys/param.h>
 #include <sys/mount.h>
 #else
-#if !defined(N_PLAT_NLM)
 #include <sys/statfs.h>
 #if !defined(sun)
 #include <mntent.h>
@@ -77,7 +76,6 @@
 #endif
 #endif
 #endif
-#endif
 
 /* Check for non-matching prototypes */
 #include "vmware.h"
@@ -100,6 +98,18 @@
 
 #if defined(__APPLE__)
 #include <sys/sysctl.h>
+
+/*
+ * F_NODIRECT was added in Mac OS 10.7.0 "Lion".  We test at runtime for the
+ * right version before using it, but we also need to get the definition.
+ */
+
+#include <sys/fcntl.h>
+
+#ifndef F_NODIRECT
+#define F_NODIRECT 62
+#endif
+
 #endif
 
 /*
@@ -369,12 +379,19 @@ FileIO_CreateFDPosix(int posix,  // IN: UNIX file descriptor
 
    FileIO_Invalidate(&fd);
 
-   if (flags & O_RDWR) {
+   switch (flags & O_ACCMODE) {
+   case O_RDWR:
       fd.flags |= (FILEIO_OPEN_ACCESS_READ | FILEIO_OPEN_ACCESS_WRITE);
-   } else if (flags & O_WRONLY) {
+      break;
+   case O_WRONLY:
       fd.flags |= FILEIO_OPEN_ACCESS_WRITE;
-   } else if (flags & O_RDONLY) {
+      break;
+   default:
+      ASSERT(FALSE);
+      /* FALLTHRU */
+   case O_RDONLY:
       fd.flags |= FILEIO_OPEN_ACCESS_READ;
+      break;
    }
 
 #if defined(O_SYNC) // Not available in FreeBSD tools build
@@ -382,6 +399,9 @@ FileIO_CreateFDPosix(int posix,  // IN: UNIX file descriptor
       fd.flags |= FILEIO_OPEN_SYNC;
    }
 #endif
+   if (flags & O_APPEND) {
+      fd.flags |= FILEIO_OPEN_APPEND;
+   }
 
    fd.posix = posix;
 
@@ -748,9 +768,10 @@ PosixFileOpener(ConstUnicode pathName,  // IN:
 /*
  *----------------------------------------------------------------------
  *
- * FileIO_Create --
+ * FileIOCreateRetry --
  *
  *      Open/create a file; specify creation mode
+ *      May perform retries to deal with certain OS conditions
  *
  * Results:
  *      FILEIO_SUCCESS on success: 'file' is set
@@ -765,11 +786,12 @@ PosixFileOpener(ConstUnicode pathName,  // IN:
  */
 
 FileIOResult
-FileIO_Create(FileIODescriptor *file,   // OUT:
-              ConstUnicode pathName,    // IN:
-              int access,               // IN:
-              FileIOOpenAction action,  // IN:
-              int mode)                 // IN: mode_t for creation
+FileIOCreateRetry(FileIODescriptor *file,   // OUT:
+                  ConstUnicode pathName,    // IN:
+                  int access,               // IN:
+                  FileIOOpenAction action,  // IN:
+                  int mode,                 // IN: mode_t for creation
+                  uint32 msecMaxWaitTime)   // IN: Ignored
 {
    uid_t uid = -1;
    int fd = -1;
@@ -793,7 +815,7 @@ FileIO_Create(FileIODescriptor *file,   // OUT:
    if (access & FILEIO_OPEN_EXCLUSIVE_LOCK_MACOS) {
       flags |= O_EXLOCK;
    }
-#elif !defined(__FreeBSD__) && !defined(sun) && !defined(N_PLAT_NLM)
+#elif !defined(__FreeBSD__) && !defined(sun)
    /*
     * If FILEIO_OPEN_EXCLUSIVE_LOCK or FILEIO_OPEN_MULTIWRITER_LOCK or
     * (FILEIO_OPEN_ACCESS_READ | FILEIO_OPEN_LOCKED) are passed, and we are
@@ -856,18 +878,36 @@ FileIO_Create(FileIODescriptor *file,   // OUT:
       flags |= O_NONBLOCK;
    }
 
+   if (access & FILEIO_OPEN_APPEND) {
+      flags |= O_APPEND;
+   }
+
+#if defined(O_NOFOLLOW)
+   if (access & FILEIO_OPEN_ACCESS_NOFOLLOW) {
+      flags |= O_NOFOLLOW;
+   }
+#endif
+
+#if defined(linux)
+   if (access & FILEIO_OPEN_SYNC) {
+      flags |= O_SYNC;
+   }
+#endif
+
+#if defined(O_NOFOLLOW)
+   if (access & FILEIO_OPEN_ACCESS_NOFOLLOW) {
+      flags |= O_NOFOLLOW;
+   }
+#endif
+
+   flags |= FileIO_OpenActions[action];
+
    file->flags = access;
 
    if (access & FILEIO_OPEN_PRIVILEGED) {
       uid = Id_BeginSuperUser();
    }
 
-   flags |= 
-#if defined(linux) && !defined(N_PLAT_NLM)
-            ((access & FILEIO_OPEN_SYNC) ? O_SYNC : 0) |
-#endif
-            FileIO_OpenActions[action];
-
    fd = PosixFileOpener(pathName, flags, mode);
 
    error = errno;
@@ -890,6 +930,20 @@ FileIO_Create(FileIODescriptor *file,   // OUT:
          ret = FileIOErrno2Result(errno);
          goto error;
       }
+
+      if (!(access & FILEIO_OPEN_SYNC)) {
+         /*
+          * F_NODIRECT was added in Mac OS 10.7.0 "Lion" which has Darwin
+          * kernel 11.0.0.
+          */
+         if (Hostinfo_OSVersion(0) >= 11) {
+            error = fcntl(fd, F_NODIRECT, 1);
+            if (error == -1) {
+               ret = FileIOErrno2Result(errno);
+               goto error;
+            }
+         }
+      }
    }
 #endif
 
@@ -924,6 +978,36 @@ error:
 }
 
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * FileIO_Create --
+ *
+ *      Open/create a file; specify creation mode
+ *
+ * Results:
+ *      FILEIO_SUCCESS on success: 'file' is set
+ *      FILEIO_OPEN_ERROR_EXIST if the file already exists
+ *      FILEIO_FILE_NOT_FOUND if the file is not present
+ *      FILEIO_ERROR for other errors
+ *
+ * Side effects:
+ *      None
+ *
+ *----------------------------------------------------------------------
+ */
+
+FileIOResult
+FileIO_Create(FileIODescriptor *file,   // OUT:
+              ConstUnicode pathName,    // IN:
+              int access,               // IN:
+              FileIOOpenAction action,  // IN:
+              int mode)                 // IN: mode_t for creation
+{
+   return FileIOCreateRetry(file, pathName, access, action, mode, 0);
+}
+
+
 /*
  *----------------------------------------------------------------------
  *
@@ -949,7 +1033,8 @@ FileIO_Open(FileIODescriptor *file,   // OUT:
             int access,               // IN:
             FileIOOpenAction action)  // IN:
 {
-   return FileIO_Create(file, pathName, access, action, S_IRUSR | S_IWUSR);
+   return FileIOCreateRetry(file, pathName, access, action,
+                            S_IRUSR | S_IWUSR, 0);
 }
 
 
@@ -972,7 +1057,7 @@ FileIO_Open(FileIODescriptor *file,   // OUT:
  *----------------------------------------------------------------------
  */
 
-#if defined(linux) && !defined(N_PLAT_NLM) && defined(SYS__llseek)
+#if defined(linux) && defined(SYS__llseek)
 /*
  * If the llseek system call exists, use it to provide a version of 64-bit
  * lseek() functionality, for FileIO_Seek() to use if appropriate.
@@ -1188,7 +1273,6 @@ FileIO_Read(FileIODescriptor *fd,  // IN:
 }
 
 
-#if !defined(N_PLAT_NLM)
 /*
  *----------------------------------------------------------------------
  *
@@ -1213,7 +1297,6 @@ FileIO_Truncate(FileIODescriptor *file,  // IN:
 
    return ftruncate(file->posix, newLength) == 0;
 }
-#endif /* !defined(N_PLAT_NLM) */
 
 
 /*
@@ -1255,7 +1338,6 @@ FileIO_Close(FileIODescriptor *file)  // IN:
 }
 
 
-#if !defined(N_PLAT_NLM)
 /*
  *----------------------------------------------------------------------
  *
@@ -1794,7 +1876,6 @@ exit:
 }
 #endif /* defined(__linux__) || defined(__APPLE__) || defined(__FreeBSD__) ||
           defined(sun) */
-#endif /* !defined(N_PLAT_NLM) */
 
 /*
  *----------------------------------------------------------------------
@@ -1846,16 +1927,12 @@ FileIO_GetAllocSize(const FileIODescriptor *fd)  // IN
 
    ASSERT(fd);
 
-#if __linux__ && defined(N_PLAT_NLM)
-   /* Netware doesn't have st_blocks.  Just fall back to GetSize. */
-   return FileIO_GetSize(fd);
-#else
    /*
     * If you don't like the magic number 512, yell at the people
     * who wrote sys/stat.h and tell them to add a #define for it.
     */
+
    return (fstat(fd->posix, &s) == -1) ? -1 : s.st_blocks * 512;
-#endif
 }
 
 
@@ -2038,10 +2115,7 @@ Bool
 FileIO_SupportsFileSize(const FileIODescriptor *fd,  // IN:
                         uint64 requestedSize)        // IN:
 {
-#if defined(N_PLAT_NLM)
-   /* API we use on NetWare cannot go over 2GB - 1 although filesystem could. */
-   return requestedSize <= 0x7FFFFFFF;
-#elif defined(linux)
+#if defined(linux)
    /*
     * Linux makes test on seek(), so we can do simple non-intrusive test.
     * Verified to work on 2.2.x, 2.4.x and 2.6.x, with ext2, ext3, smbfs, 
@@ -2243,7 +2317,7 @@ FileIO_DescriptorToStream(FileIODescriptor *fdesc,  // IN:
  *
  *      Returns TRUE if the host OS is new enough to support F_PREALLOCATE
  *      without data loss bugs.  On OSX, this has been verified fixed
- *      on 10.6 build with kern.osreleasae 10.0.0d6.
+ *      on 10.6 build with kern.osrelease 10.0.0d6.
  *
  * Results:
  *      TRUE if the current host OS is new enough.
@@ -2258,6 +2332,8 @@ FileIO_DescriptorToStream(FileIODescriptor *fdesc,  // IN:
 static Bool
 HostSupportsPrealloc(void)
 {
+   static Atomic_uint32 supported = { 0 };
+   enum { PREALLOC_UNKNOWN = 0, PREALLOC_YES, PREALLOC_NO } val;
    char curRel[32];
    char type;
    unsigned static const int req[] = { 10, 0, 0, 6 };
@@ -2266,6 +2342,11 @@ HostSupportsPrealloc(void)
    size_t len = sizeof(curRel);
    Bool ret = FALSE;
 
+   val = Atomic_Read(&supported);
+   if (val != PREALLOC_UNKNOWN) {
+      return val == PREALLOC_YES;
+   }
+
    if (sysctlbyname("kern.osrelease", (void *) &curRel, &len, NULL, 0) == -1) {
       goto exit;
    }
@@ -2315,10 +2396,12 @@ HostSupportsPrealloc(void)
 exit:
    if (!ret && filePosixOptions.initialized && 
        filePosixOptions.aioNumThreads == 1) {
-      ret =TRUE;
+      ret = TRUE;
    }
 
-   return  ret;
+   Atomic_Write(&supported, ret ? PREALLOC_YES : PREALLOC_NO);
+
+   return ret;
 }
 
 #else
