@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 1998-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 1998-2011 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -42,7 +42,6 @@
 #include "vmware.h"
 #include "util.h"
 #include "str.h"
-#include "strutil.h"
 #include "posix.h"
 #include "file.h"
 
@@ -67,7 +66,7 @@
  */
 
 int64
-File_GetModTime(const char *pathName)  // IN:
+File_GetModTime(ConstUnicode pathName)  // IN:
 {
    int64 theTime;
    struct stat statbuf;
@@ -92,7 +91,7 @@ File_GetModTime(const char *pathName)  // IN:
  *
  * Results:
  *      As described.
- *
+ *      
  * Side effects:
  *      None.
  *
@@ -100,7 +99,7 @@ File_GetModTime(const char *pathName)  // IN:
  */
 
 UnicodeIndex
-FileFirstSlashIndex(const char *pathName,     // IN:
+FileFirstSlashIndex(ConstUnicode pathName,    // IN:
                     UnicodeIndex startIndex)  // IN:
 {
    UnicodeIndex firstFS;
@@ -139,7 +138,7 @@ FileFirstSlashIndex(const char *pathName,     // IN:
  *
  * Results:
  *      As described.
- *
+ *      
  * Side effects:
  *      None.
  *
@@ -147,7 +146,7 @@ FileFirstSlashIndex(const char *pathName,     // IN:
  */
 
 static UnicodeIndex
-FileLastSlashIndex(const char *pathName,     // IN:
+FileLastSlashIndex(ConstUnicode pathName,    // IN:
                    UnicodeIndex startIndex)  // IN:
 {
    UnicodeIndex lastFS;
@@ -203,7 +202,7 @@ FileLastSlashIndex(const char *pathName,     // IN:
  *
  * Results:
  *      As described.
- *
+ *      
  * Side effects:
  *      None.
  *
@@ -211,14 +210,14 @@ FileLastSlashIndex(const char *pathName,     // IN:
  */
 
 void
-File_SplitName(const char *pathName,  // IN:
-               char **volume,         // OUT/OPT:
-               char **directory,      // OUT/OPT:
-               char **base)           // OUT/OPT:
+File_SplitName(ConstUnicode pathName,  // IN:
+               Unicode *volume,        // OUT (OPT):
+               Unicode *directory,     // OUT (OPT):
+               Unicode *base)          // OUT (OPT):
 {
-   char *vol;
-   char *dir;
-   char *bas;
+   Unicode vol;
+   Unicode dir;
+   Unicode bas;
    UnicodeIndex volEnd;
    UnicodeIndex length;
    UnicodeIndex baseBegin;
@@ -235,8 +234,8 @@ File_SplitName(const char *pathName,  // IN:
 #if defined(_WIN32)
    pathLen = Unicode_LengthInCodePoints(pathName);
    if ((pathLen > 2) &&
-       (StrUtil_StartsWith(pathName, "\\\\") ||
-        StrUtil_StartsWith(pathName, "//"))) {
+       (Unicode_StartsWith(pathName, "\\\\") ||
+        Unicode_StartsWith(pathName, "//"))) {
       /* UNC path */
       volEnd = FileFirstSlashIndex(pathName, 2);
 
@@ -299,19 +298,19 @@ File_SplitName(const char *pathName,  // IN:
    if (volume) {
       *volume = vol;
    } else {
-      free(vol);
+      Unicode_Free(vol);
    }
 
    if (directory) {
       *directory = dir;
    } else {
-      free(dir);
+      Unicode_Free(dir);
    }
 
    if (base) {
       *base = bas;
    } else {
-      free(bas);
+      Unicode_Free(bas);
    }
 }
 
@@ -321,83 +320,35 @@ File_SplitName(const char *pathName,  // IN:
  *
  * File_PathJoin --
  *
- *      Join the dirName and baseName together to create a (full) path.
+ *      Join the dirName and baseName together to create a (full) path but
+ *      don't add a DIRSEPS unless necessary.
  *
- *      This code concatenates two strings together and omits a redundant
- *      directory separator between the two.
+ *      File_PathJoin("a", "b")  -> "a/b"
+ *      File_PathJoin("a/", "b") -> "a/b"
  *
- *      On Windows, the 'baseName' argument may not be a fully qualified path.
- *      That is, it may not be an absolute path containing a drive letter nor
- *      may it be a UNC path.
- *
- * Examples:
- *      File_PathJoin("", "b")            -> "/b"
- *      File_PathJoin("/", "b")           -> "/b"
- *      File_PathJoin("a", "b")           -> "a/b"
- *      File_PathJoin("a/", "b")          -> "a/b"
- *      File_PathJoin("a/////", "b")      -> "a/b"
- *      File_PathJoin("a", "")            -> "a/"
- *      File_PathJoin("a", "/")           -> "a/"
- *      File_PathJoin("a", "/b")          -> "a/b"
- *      File_PathJoin("a", "/////b")      -> "a/b" (only posix)
- *      File_PathJoin("a/", "/b")         -> "a/b"
- *      File_PathJoin("a/////", "/////b") -> "a/b" (only posix)
- *
- * Results:
+ * Results: 
  *      The constructed path which must be freed by the caller.
  *
- * Side effects:
+ * Side effects: 
  *      None
  *
  *---------------------------------------------------------------------------
  */
 
-char *
-File_PathJoin(const char *dirName,   // IN:
-              const char *baseName)  // IN: See above.
+Unicode
+File_PathJoin(ConstUnicode dirName,   // IN:
+              ConstUnicode baseName)  // IN:
 {
-   char *result;
-   char *newDir = NULL;
+   Unicode result;
 
    ASSERT(dirName);
    ASSERT(baseName);
 
-   /*
-    * Remove ALL directory separators from baseName begin.
-    */
-#if defined(_WIN32)
-   {
-      const char *oldBaseName = baseName;
-
-      /*
-       * Reject drive letters in baseName.
-       */
-      ASSERT(Unicode_LengthInCodePoints(baseName) < 2 ||
-             Unicode_FindSubstrInRange(baseName, 1, 1, ":", 0, 1) ==
-             UNICODE_INDEX_NOT_FOUND);
-
-      while (*baseName == '/' || *baseName == '\\') {
-         baseName++;
-      }
-
-      /*
-       * Reject UNC paths for baseName.
-       */
-      ASSERT(baseName - oldBaseName < 2);
+   if (Unicode_EndsWith(dirName, DIRSEPS)) {
+      result = Unicode_Append(dirName, baseName);
+   } else {
+      result = Unicode_Join(dirName, DIRSEPS, baseName, NULL);
    }
-#else
-   while (*baseName == '/') {
-      baseName++;
-   }
-#endif
-
-   /*
-    * Remove ALL directory separators from dirName end.
-    */
-   newDir = File_StripSlashes(dirName);
-
-   result = Unicode_Join(newDir, DIRSEPS, baseName, NULL);
-   free(newDir);
 
    return result;
 }
@@ -421,28 +372,28 @@ File_PathJoin(const char *dirName,   // IN:
  *      A NULL pointer may be passed for one or more OUT parameters,
  *      in which case that parameter is not returned.
  *
- * Results:
+ * Results: 
  *      As described.
  *
- * Side effects:
+ * Side effects: 
  *      The return values must be freed.
  *
  *---------------------------------------------------------------------------
  */
 
-void
-File_GetPathName(const char *fullPath,  // IN:
-                 char **pathName,       // OUT/OPT:
-                 char **baseName)       // OUT/OPT:
+void 
+File_GetPathName(ConstUnicode fullPath,  // IN:
+                 Unicode *pathName,      // OUT (OPT):
+                 Unicode *baseName)      // OUT (OPT):
 {
-   char *volume;
+   Unicode volume;
    UnicodeIndex len;
    UnicodeIndex curLen;
 
    File_SplitName(fullPath, &volume, pathName, baseName);
 
    if (pathName == NULL) {
-      free(volume);
+      Unicode_Free(volume);
       return;
    }
 
@@ -451,12 +402,12 @@ File_GetPathName(const char *fullPath,  // IN:
     */
 
    if (!Unicode_IsEmpty(volume)) {
-      char *temp = Unicode_Append(volume, *pathName);
+      Unicode temp = Unicode_Append(volume, *pathName);
 
-      free(*pathName);
+      Unicode_Free(*pathName);
       *pathName = temp;
    }
-   free(volume);
+   Unicode_Free(volume);
 
    /*
     * Remove any trailing directory separator characters.
@@ -472,9 +423,9 @@ File_GetPathName(const char *fullPath,  // IN:
    }
 
    if (curLen < len) {
-      char *temp = Unicode_Substr(*pathName, 0, curLen);
+      Unicode temp = Unicode_Substr(*pathName, 0, curLen);
 
-      free(*pathName);
+      Unicode_Free(*pathName);
       *pathName = temp;
    }
 }
@@ -496,13 +447,10 @@ File_GetPathName(const char *fullPath,  // IN:
  *----------------------------------------------------------------------
  */
 
-char *
-File_StripSlashes(const char *path)  // IN:
+Unicode
+File_StripSlashes(ConstUnicode path)  // IN:
 {
-   char *result;
-   char *volume;
-   char *dir;
-   char *base;
+   Unicode result, volume, dir, base;
 
    /*
     * SplitName handles all drive letter/UNC/whatever cases, all we
@@ -530,16 +478,16 @@ File_StripSlashes(const char *path)  // IN:
          i--;
       }
 
-      free(dir);
+      Unicode_Free(dir);
       dir = Unicode_AllocWithLength(dir2, i, STRING_ENCODING_UTF8);
       free(dir2);
    }
 
    result = Unicode_Join(volume, dir, base, NULL);
 
-   free(volume);
-   free(dir);
-   free(base);
+   Unicode_Free(volume);
+   Unicode_Free(dir);
+   Unicode_Free(base);
 
    return result;
 }
@@ -718,28 +666,28 @@ File_PrependToPath(const char *searchPath,  // IN:
  *-----------------------------------------------------------------------------
  */
 
-char *
-File_ReplaceExtension(const char *pathName,      // IN:
-                      const char *newExtension,  // IN:
-                      uint32 numExtensions,      // IN:
-                      ...)                       // IN:
+Unicode
+File_ReplaceExtension(ConstUnicode pathName,      // IN:
+                      ConstUnicode newExtension,  // IN:
+                      uint32 numExtensions,       // IN:
+                      ...)                        // IN:
 {
-   char *path;
-   char *base;
-   char *result;
+   Unicode path;
+   Unicode base;
+   Unicode result;
    va_list arguments;
    UnicodeIndex index;
-
+   
    ASSERT(pathName);
    ASSERT(newExtension);
-   ASSERT(*newExtension == '.');
+   ASSERT(Unicode_StartsWith(newExtension, "."));
 
    File_GetPathName(pathName, &path, &base);
 
    index = Unicode_FindLast(base, ".");
 
    if (index != UNICODE_INDEX_NOT_FOUND) {
-      char *oldBase = base;
+      Unicode oldBase = base;
 
       if (numExtensions) {
          uint32 i;
@@ -752,9 +700,9 @@ File_ReplaceExtension(const char *pathName,      // IN:
          va_start(arguments, numExtensions);
 
          for (i = 0; i < numExtensions ; i++) {
-            char *oldExtension = va_arg(arguments, char *);
+            Unicode oldExtension = va_arg(arguments, Unicode);
 
-            ASSERT(*oldExtension == '.');
+            ASSERT(Unicode_StartsWith(oldExtension, "."));
 
             if (Unicode_CompareRange(base, index, -1,
                                      oldExtension, 0, -1, FALSE) == 0) {
@@ -770,7 +718,7 @@ File_ReplaceExtension(const char *pathName,      // IN:
       }
 
       if (oldBase != base) {
-         free(oldBase);
+         Unicode_Free(oldBase);
       }
    }
 
@@ -780,8 +728,8 @@ File_ReplaceExtension(const char *pathName,      // IN:
       result = Unicode_Join(path, DIRSEPS, base, newExtension, NULL);
    }
 
-   free(path);
-   free(base);
+   Unicode_Free(path);
+   Unicode_Free(base);
 
    return result;
 }
@@ -805,8 +753,8 @@ File_ReplaceExtension(const char *pathName,      // IN:
  *-----------------------------------------------------------------------------
  */
 
-char *
-File_RemoveExtension(const char *pathName)  // IN:
+Unicode
+File_RemoveExtension(ConstUnicode pathName)  // IN:
 {
    UnicodeIndex index;
 
