@@ -79,6 +79,50 @@ HgfsNotify_Shutdown(void)
 }
 
 
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * HgfsNotify_Suspend --
+ *
+ *    Suspends sending notifications.
+ *
+ * Results:
+ *    None.
+ *
+ * Side effects:
+ *    None.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+void
+HgfsNotify_Suspend(void)
+{
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * HgfsNotify_Resume --
+ *
+ *    Resumes sending notifications.
+ *
+ * Results:
+ *    None.
+ *
+ * Side effects:
+ *    None.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+void
+HgfsNotify_Resume(void)
+{
+}
+
+
 /*
  *-----------------------------------------------------------------------------
  *
