@@ -98,20 +98,24 @@ extern char *Hostinfo_GetLibraryPath(void *addr);
 
 #if !defined(_WIN32)
 extern void Hostinfo_ResetProcessState(const int *keepFds, size_t numKeepFds);
-extern int Hostinfo_Execute(const char *command, char * const *args,
-			    Bool wait);
+extern int Hostinfo_Execute(const char *path,
+                            char * const *args,
+                            Bool wait,
+                            const int *keepFds,
+                            size_t numKeepFds);
 typedef enum HostinfoDaemonizeFlags {
    HOSTINFO_DAEMONIZE_DEFAULT = 0,
    HOSTINFO_DAEMONIZE_NOCHDIR = (1 << 0),
    HOSTINFO_DAEMONIZE_NOCLOSE = (1 << 1),
    HOSTINFO_DAEMONIZE_EXIT    = (1 << 2),
+   HOSTINFO_DAEMONIZE_LOCKPID = (1 << 3),
 } HostinfoDaemonizeFlags;
 extern Bool Hostinfo_Daemonize(const char *path,
                                char * const *args,
                                HostinfoDaemonizeFlags flags,
                                const char *pidPath,
-                               const int *openFds,
-                               size_t numFds);
+                               const int *keepFds,
+                               size_t numKeepFds);
 #endif
 
 extern Unicode Hostinfo_GetUser(void);
@@ -202,4 +206,12 @@ Bool Hostinfo_GetLoadAverage(uint32 *l);
 size_t Hostinfo_GetKernelZoneElemSize(char const *name);
 #endif
 
+#ifdef _WIN32
+static INLINE Bool
+Hostinfo_AtLeastVista(void)
+{
+   return (Hostinfo_GetOSType() >= OS_VISTA);
+}
+#endif
+
 #endif /* ifndef _HOSTINFO_H_ */
