@@ -34,32 +34,29 @@
  * used system logger values will be invalid and the errant usage caught.
  */
 
-#define VMW_LOG_BASE      100
-#define VMW_LOG_PANIC     VMW_LOG_BASE +  0  // highest priority
-#define VMW_LOG_ERROR     VMW_LOG_BASE +  1
-#define VMW_LOG_WARNING   VMW_LOG_BASE +  2  // <= goes to stderr by default
-#define VMW_LOG_INFO      VMW_LOG_BASE +  3  // <= goes to log by default
-#define VMW_LOG_VERBOSE   VMW_LOG_BASE +  4
-#define VMW_LOG_TRIVIA    VMW_LOG_BASE +  5
-#define VMW_LOG_DEBUG_00  VMW_LOG_BASE +  6  // noisiest level of debugging
-#define VMW_LOG_DEBUG_01  VMW_LOG_BASE +  7
-#define VMW_LOG_DEBUG_02  VMW_LOG_BASE +  8
-#define VMW_LOG_DEBUG_03  VMW_LOG_BASE +  9
-#define VMW_LOG_DEBUG_04  VMW_LOG_BASE + 10
-#define VMW_LOG_DEBUG_05  VMW_LOG_BASE + 11
-#define VMW_LOG_DEBUG_06  VMW_LOG_BASE + 12
-#define VMW_LOG_DEBUG_07  VMW_LOG_BASE + 13
-#define VMW_LOG_DEBUG_08  VMW_LOG_BASE + 14
-#define VMW_LOG_DEBUG_09  VMW_LOG_BASE + 15
-#define VMW_LOG_DEBUG_10  VMW_LOG_BASE + 16  // lowest priority; least noisy
+#define VMW_LOG_BASE     100
+#define VMW_LOG_PANIC    (VMW_LOG_BASE     +  0) // highest priority
+#define VMW_LOG_ERROR    (VMW_LOG_BASE     +  5)
+#define VMW_LOG_WARNING  (VMW_LOG_BASE     + 10) // <= goes to stderr by default
+#define VMW_LOG_AUDIT    (VMW_LOG_BASE     + 15) // *ALWAYS* output to the log
+#define VMW_LOG_INFO     (VMW_LOG_BASE     + 20) // <= goes to log by default
+#define VMW_LOG_VERBOSE  (VMW_LOG_BASE     + 25)
+#define VMW_LOG_TRIVIA   (VMW_LOG_BASE     + 30) 
+#define VMW_LOG_DEBUG_00 (VMW_LOG_BASE     + 35) // noisiest level of debugging
+#define VMW_LOG_DEBUG_01 (VMW_LOG_DEBUG_00 +  1)
+#define VMW_LOG_DEBUG_02 (VMW_LOG_DEBUG_00 +  2)
+#define VMW_LOG_DEBUG_03 (VMW_LOG_DEBUG_00 +  3)
+#define VMW_LOG_DEBUG_04 (VMW_LOG_DEBUG_00 +  4)
+#define VMW_LOG_DEBUG_05 (VMW_LOG_DEBUG_00 +  5)
+#define VMW_LOG_DEBUG_06 (VMW_LOG_DEBUG_00 +  6)
+#define VMW_LOG_DEBUG_07 (VMW_LOG_DEBUG_00 +  7)
+#define VMW_LOG_DEBUG_08 (VMW_LOG_DEBUG_00 +  8)
+#define VMW_LOG_DEBUG_09 (VMW_LOG_DEBUG_00 +  9)
+#define VMW_LOG_DEBUG_10 (VMW_LOG_DEBUG_00 + 10) // lowest priority; least noisy
 
-void
-LogVNoRoute(int rawLevel,
-            const char *fmt,
-            va_list args);
+#define VMW_LOG_NO_ROUTE 0x80000000  // Force routing to Log
 
-
-void LogV(int level,
+void LogV(uint32 routing,
           const char *fmt,
           va_list args);
 
@@ -74,23 +71,23 @@ void LogV(int level,
  */
 
 static INLINE void PRINTF_DECL(2, 3)
-Log_Level(int level,
+Log_Level(uint32 routing,
           const char *fmt,
           ...)
 {
    va_list ap;
 
    va_start(ap, fmt);
-   LogV(level, fmt, ap);
+   LogV(routing, fmt, ap);
    va_end(ap);
 }
 
 
 static INLINE void
-Log_String(int level,
+Log_String(uint32 routing,
            const char *string)
 {
-   Log_Level(level, "%s", string);
+   Log_Level(routing, "%s", string);
 }
 
 
@@ -106,6 +103,18 @@ Log_Panic(const char *fmt,
 }
 
 
+static INLINE void PRINTF_DECL(1, 2)
+Log_Audit(const char *fmt,
+          ...)
+{
+   va_list ap;
+
+   va_start(ap, fmt);
+   LogV(VMW_LOG_AUDIT, fmt, ap);
+   va_end(ap);
+}
+
+
 static INLINE void PRINTF_DECL(1, 2)
 Log_Error(const char *fmt,
           ...)
@@ -142,8 +151,13 @@ Log_Trivia(const char *fmt,
 }
 
 #if !defined(VMM)
-typedef void (LogBasicFunc)(const char *fmt,
-                            va_list args);
+
+/* Forward decl */
+struct MsgList;
+
+typedef void (LogOutputFunc)(int level,
+                             const char *fmt,
+                             va_list args);
 
 
 typedef enum {
@@ -218,17 +232,17 @@ Bool Log_RemoveFile(Bool alwaysRemove);
 void Log_DisableThrottling(void);
 void Log_EnableStderrWarnings(Bool stderrOutput);
 void Log_BackupOldFiles(const char *fileName, Bool noRename);
-Bool Log_CopyFile(const char *fileName);
+Bool Log_CopyFile(const char *fileName, struct MsgList **errs);
 uint32 Log_MaxLineLength(void);
 
-void Log_RegisterBasicFunctions(LogBasicFunc *log,
-                                LogBasicFunc *warning);
+void Log_RegisterOutputFunction(LogOutputFunc *func);
 
 Bool Log_SetOutput(const char *fileName,
                    const char *config,
                    Bool copy,
                    uint32 systemLoggerUse,
-                   char *systemLoggerID);
+                   char *systemLoggerID,
+                   struct MsgList **errs);
 
 size_t Log_MakeTimeString(Bool millisec,
                           char *buf,
