@@ -237,9 +237,12 @@
    LOGLEVEL_VAR(sg), /* lib/sg */ \
    LOGLEVEL_VAR(ftcpt), \
    LOGLEVEL_VAR(wrapLib),  \
-   LOGLEVEL_VAR(bluetooth),  \
    LOGLEVEL_VAR(digestlib), \
+   LOGLEVEL_VAR(inputdevtap), \
    LOGLEVEL_VAR(svgadevtap), \
+   LOGLEVEL_VAR(masReceipt), /* lib/masReceipt */ \
+   LOGLEVEL_VAR(serviceImpl), /* lib/serviceImpl */ \
+   LOGLEVEL_VAR(serviceUser), /* lib/serviceUser */ \
    LOGLEVEL_VAR(ssl), \
    LOGLEVEL_VAR(vmrc), \
    /* end of list */
