@@ -314,35 +314,6 @@ struct VixCommandSSPI {
 #include "vmware_pack_end.h"
 VixCommandSSPI;
 
-/*
- * **********************************************************
- * Open VM Command
- *
- */
-
-typedef
-#include "vmware_pack_begin.h"
-struct VixMsgVMOpenRequest {
-   VixCommandRequestHeader header;
-   int32                   options; //options for VM_Open or CreateWorkingCopy
-   uint32                  xmlPathNameSize;
-   uint32                  vmxPathNameSize;
-   //this is followed by the xml pathname and the vmx pathname
-}
-#include "vmware_pack_end.h"
-VixMsgVMOpenRequest;
-
-typedef
-#include "vmware_pack_begin.h"
-struct VixMsgVMOpenResponse {
-   VixCommandResponseHeader   header;
-   int32                      vmPowerState;
-   uint32                     vmxPathNameSize;
-}
-#include "vmware_pack_end.h"
-VixMsgVMOpenResponse;
-
-
 
 /*
  * **********************************************************
@@ -1321,22 +1292,6 @@ struct VixMsgSetSharedFolderRequest {
 VixMsgSetSharedFolderRequest;
 
 
-/*
- * **********************************************************
- * Open a URL in the guest.
- */
-typedef
-#include "vmware_pack_begin.h"
-struct VixMsgOpenUrlRequest {
-   VixCommandRequestHeader  header;
-
-   int32                    windowState;
-   uint64                   urlLength;
-}
-#include "vmware_pack_end.h"
-VixMsgOpenUrlRequest;
-
-
 /*
  * **********************************************************
  * Capture the screen of a VM
@@ -1694,29 +1649,6 @@ struct VixMsgMountHGFSRequest {
 VixMsgMountHGFSRequest;
 
 
-/*
- * **********************************************************
- * Get the state of all USB devices.
- */
-
-
-/*
- * This is used to reply to VixMsgGetUSBDeviceListRequest operations.
- */
-typedef
-#include "vmware_pack_begin.h"
-struct VixMsgGetUSBDeviceListResponse {
-   VixCommandResponseHeader   header;
-
-   int32                      numRunningDevices;
-   uint32                     runningDeviceListLength;
-
-   uint32                     patternListLength;
-}
-#include "vmware_pack_end.h"
-VixMsgGetUSBDeviceListResponse;
-
-
 /*
  * Get guest networking config
  */
@@ -2138,7 +2070,7 @@ enum {
    /* DEPRECATED VIX_COMMAND_SYNCDRIVER_THAW                  = 34, */
    /* DEPRECATED VIX_COMMAND_HOT_ADD_DISK                     = 35, */
    /* DEPRECATED VIX_COMMAND_HOT_REMOVE_DISK                  = 36, */
-   VIX_COMMAND_SET_GUEST_PRINTER                = 37,
+   /* DEPRECATED VIX_COMMAND_SET_GUEST_PRINTER                = 37, */
    VIX_COMMAND_WAIT_FOR_TOOLS                   = 38,
    VIX_COMMAND_CREATE_RUNNING_VM_SNAPSHOT       = 39,
    VIX_COMMAND_CONSOLIDATE_RUNNING_VM_SNAPSHOT  = 40,
@@ -2150,16 +2082,16 @@ enum {
    VIX_COMMAND_RUN_SCRIPT_IN_GUEST              = 46,
    VIX_COMMAND_OPEN_VM                          = 47,
    /* DEPRECATED VIX_COMMAND_GET_DISK_PROPERTIES              = 48, */
-   VIX_COMMAND_OPEN_URL                         = 49,
+   /* DEPRECATED VIX_COMMAND_OPEN_URL                         = 49, */
    VIX_COMMAND_GET_HANDLE_STATE                 = 50,
    /* DEPRECATED VIX_COMMAND_SET_HANDLE_STATE                 = 51, */
    VIX_COMMAND_CREATE_WORKING_COPY              = 55, // DELETE this when we switch remote foundry to VIM
    VIX_COMMAND_DISCARD_WORKING_COPY             = 56, // DELETE this when we switch remote foundry to VIM
    VIX_COMMAND_SAVE_WORKING_COPY                = 57, // DELETE this when we switch remote foundry to VIM
    VIX_COMMAND_CAPTURE_SCREEN                   = 58,
-   VIX_COMMAND_GET_VMDB_VALUES                  = 59,
-   VIX_COMMAND_SET_VMDB_VALUES                  = 60,
-   VIX_COMMAND_READ_XML_FILE                    = 61,
+   /* DEPRECATED VIX_COMMAND_GET_VMDB_VALUES                  = 59, */
+   /* DEPRECATED VIX_COMMAND_SET_VMDB_VALUES                  = 60, */
+   /* DEPRECATED VIX_COMMAND_READ_XML_FILE                    = 61, */
    VIX_COMMAND_GET_TOOLS_STATE                  = 62,
    VIX_COMMAND_CHANGE_SCREEN_RESOLUTION         = 69,
    VIX_COMMAND_DIRECTORY_EXISTS                 = 70,
@@ -2199,7 +2131,7 @@ enum {
    VIX_COMMAND_VPROBE_LOAD                      = 107,
    VIX_COMMAND_VPROBE_RESET                     = 108,
 
-   VIX_COMMAND_LIST_USB_DEVICES                 = 109,
+   /* DEPRECATED VIX_COMMAND_LIST_USB_DEVICES                 = 109, */
    VIX_COMMAND_CONNECT_HOST                     = 110,
 
    VIX_COMMAND_CREATE_LINKED_CLONE              = 112,
@@ -2370,7 +2302,7 @@ enum {
  */
 enum VixToolsFeatures {
    VIX_TOOLSFEATURE_SUPPORT_GET_HANDLE_STATE   = 0x0001,
-   VIX_TOOLSFEATURE_SUPPORT_OPEN_URL           = 0x0002,
+   /* VIX_TOOLSFEATURE_SUPPORT_OPEN_URL           = 0x0002, Removed in version 1.11*/
 };
 
 
@@ -2385,10 +2317,8 @@ enum  {
  * to the VMX.
  */
 #define VIX_BACKDOORCOMMAND_RUN_PROGRAM_DONE    "Run_Program_Done"
-#define VIX_BACKDOORCOMMAND_PROXY_MESSAGE       "VIX_Proxy_Msg"
 
 
-#define VIX_HOST_FOR_THIS_GUEST_OS "self"
 #define VIX_FEATURE_UNKNOWN_VALUE "Unknown"
 
 
