@@ -465,7 +465,8 @@ typedef uint64    BA;
 #endif
 typedef uint32    BPN;
 typedef uint32    PageNum;
-typedef unsigned  MemHandle;
+typedef unsigned      MemHandle;
+typedef unsigned int  IoHandle;
 typedef int32     World_ID;
 
 /* !! do not alter the definition of INVALID_WORLD_ID without ensuring
@@ -695,7 +696,7 @@ typedef void * UserVA;
  * that gcc will not do inlining.
  */
 
-#if defined(__GNUC__) && defined(VMM)
+#if defined(__GNUC__) && (defined(VMM) || defined (VMKERNEL))
 #define ABSOLUTELY_NOINLINE __attribute__((__noinline__))
 #endif
 
@@ -804,6 +805,16 @@ typedef void * UserVA;
 # define REGPARM
 #endif
 
+
+/*
+ * gcc can warn us if we're ignoring returns
+ */
+#if defined(__GNUC__)
+# define MUST_CHECK_RETURN __attribute__((warn_unused_result))
+#else
+# define MUST_CHECK_RETURN
+#endif
+
 /*
  * ALIGNED specifies minimum alignment in "n" bytes.
  */
