@@ -48,6 +48,7 @@
  * level. To set options, use the value of VMCISock_GetAFValue for
  * 'level' and these constants for the optname.
  */
+
 #define SO_VMCI_BUFFER_SIZE                 0
 #define SO_VMCI_BUFFER_MIN_SIZE             1
 #define SO_VMCI_BUFFER_MAX_SIZE             2
@@ -62,9 +63,25 @@
  * the current guest (or the host, if running outside a guest), while the
  * second indicates any available port.
  */
-#define VMADDR_CID_ANY  ((unsigned int) -1)
-#define VMADDR_PORT_ANY ((unsigned int) -1)
 
+#define VMADDR_CID_ANY  ((unsigned int)-1)
+#define VMADDR_PORT_ANY ((unsigned int)-1)
+
+
+/* Invalid VMCI sockets version. */
+
+#define VMCI_SOCKETS_INVALID_VERSION ((unsigned int)-1)
+
+/*
+ * Use these macros with the result of VMCISock_Version() to get the
+ * component parts of a valid version.
+ */
+
+#define VMCI_SOCKETS_VERSION_EPOCH(_v) (((_v) & 0xFF000000) >> 24)
+#define VMCI_SOCKETS_VERSION_MAJOR(_v) (((_v) & 0x00FF0000) >> 16)
+#define VMCI_SOCKETS_VERSION_MINOR(_v) (((_v) & 0x0000FFFF))
+
+/* Missing types for some platforms. */
 
 #if defined(_WIN32) || defined(VMKERNEL)
    typedef unsigned short sa_family_t;
@@ -106,22 +123,33 @@ struct sockaddr_vm {
 #  if !defined(NT_INCLUDED)
 #     include <winioctl.h>
 #     define VMCI_SOCKETS_DEVICE          L"\\\\.\\VMCI"
+#     define VMCI_SOCKETS_VERSION         0x81032058
 #     define VMCI_SOCKETS_GET_AF_VALUE    0x81032068
 #     define VMCI_SOCKETS_GET_LOCAL_CID   0x8103206c
-      static __inline int VMCISock_GetAFValue(void)
+
+      static __inline unsigned int __VMCISock_DeviceIoControl(DWORD cmd)
       {
-         int afvalue = -1;
+         unsigned int val = (unsigned int)-1;
          HANDLE device = CreateFileW(VMCI_SOCKETS_DEVICE, GENERIC_READ, 0, NULL,
                                      OPEN_EXISTING, FILE_FLAG_OVERLAPPED, NULL);
          if (INVALID_HANDLE_VALUE != device) {
             DWORD ioReturn;
-            DeviceIoControl(device, VMCI_SOCKETS_GET_AF_VALUE, &afvalue,
-                            sizeof afvalue, &afvalue, sizeof afvalue,
+            DeviceIoControl(device, cmd, &val, sizeof val, &val, sizeof val,
                             &ioReturn, NULL);
             CloseHandle(device);
             device = INVALID_HANDLE_VALUE;
          }
-         return afvalue;
+         return val;
+      }
+
+      static __inline unsigned int VMCISock_Version(void)
+      {
+         return __VMCISock_DeviceIoControl(VMCI_SOCKETS_VERSION);
+      }
+
+      static __inline int VMCISock_GetAFValue(void)
+      {
+         return (int)__VMCISock_DeviceIoControl(VMCI_SOCKETS_GET_AF_VALUE);
       }
 
       static __inline int VMCISock_GetAFValueFd(int *outFd)
@@ -137,18 +165,7 @@ struct sockaddr_vm {
 
       static __inline unsigned int VMCISock_GetLocalCID(void)
       {
-         unsigned int cid = VMADDR_CID_ANY;
-         HANDLE device = CreateFileW(VMCI_SOCKETS_DEVICE, GENERIC_READ, 0, NULL,
-                                     OPEN_EXISTING, FILE_FLAG_OVERLAPPED, NULL);
-         if (INVALID_HANDLE_VALUE != device) {
-            DWORD ioReturn;
-            DeviceIoControl(device, VMCI_SOCKETS_GET_LOCAL_CID, &cid,
-                            sizeof cid, &cid, sizeof cid, &ioReturn,
-                            NULL);
-            CloseHandle(device);
-            device = INVALID_HANDLE_VALUE;
-         }
-         return cid;
+         return __VMCISock_DeviceIoControl(VMCI_SOCKETS_GET_LOCAL_CID);
       }
 #  endif // !NT_INCLUDED
 #else // _WIN32
@@ -173,14 +190,37 @@ struct sockaddr_vm {
 #  define VMCI_SOCKETS_CLASSIC_ESX_DEVICE  "/vmfs/devices/char/vsock/vsock"
 
 # if defined(linux)
+#  define VMCI_SOCKETS_VERSION       1972
 #  define VMCI_SOCKETS_GET_AF_VALUE  1976
 #  define VMCI_SOCKETS_GET_LOCAL_CID 1977
 # elif defined(__APPLE__)
 #  include <sys/ioccom.h>
+#  define VMCI_SOCKETS_VERSION       _IOR('V', 21,  unsigned)
 #  define VMCI_SOCKETS_GET_AF_VALUE  _IOR('V', 25 , int)
 #  define VMCI_SOCKETS_GET_LOCAL_CID _IOR('V', 26 , unsigned)
 #endif
 
+   static inline unsigned int VMCISock_Version(void)
+   {
+      int fd;
+      unsigned int version;
+
+      fd = open(VMCI_SOCKETS_DEFAULT_DEVICE, O_RDWR);
+      if (fd < 0) {
+         fd = open(VMCI_SOCKETS_CLASSIC_ESX_DEVICE, O_RDWR);
+         if (fd < 0) {
+            return VMCI_SOCKETS_INVALID_VERSION;
+         }
+      }
+
+      if (ioctl(fd, VMCI_SOCKETS_VERSION, &version) < 0) {
+         version = VMCI_SOCKETS_INVALID_VERSION;
+      }
+
+      close(fd);
+      return version;
+   }
+
    /*
     *----------------------------------------------------------------------------
     *
