@@ -68,6 +68,36 @@
    g_source_attach(__src, g_main_loop_get_context((ctx)->mainLoop));    \
 } while (0)
 
+/* Indentation levels for the state log function below. */
+#define TOOLS_STATE_LOG_ROOT        0
+#define TOOLS_STATE_LOG_CONTAINER   1
+#define TOOLS_STATE_LOG_PLUGIN      2
+
+/**
+ * Convenience function for printing state logs. This function makes sure
+ * all code logging state information uses the same log domain, level and
+ * use the same indentation.
+ *
+ * @param[in] level  Indentation level (see constants above).
+ * @param[in] fmt    Message format.
+ * @param[in] ...    Message arguments.
+ */
+
+static inline void
+ToolsCore_LogState(guint level,
+                   const char *fmt,
+                   ...)
+{
+   gchar *indented = g_strdup_printf("%*s%s", 3 * level, "", fmt);
+
+   va_list args;
+   va_start(args, fmt);
+   g_logv("state", G_LOG_LEVEL_INFO, indented, args);
+   va_end(args);
+
+   g_free(indented);
+}
+
 
 /**
  * Signal sent when registering or unregistering capabilities.
@@ -166,6 +196,15 @@
 
 #endif
 
+/**
+ * @brief Property where the container's ToolsAppCtx is stored.
+ *
+ * This property is useful in cases where the client code has access to the
+ * service object instance but not to the container's context object, such as
+ * in the callback for the object's "notify" signal.
+ */
+#define TOOLS_CORE_PROP_CTX "tcs_app_ctx"
+
 
 /**
  * This enum lists all API versions that different versions of vmtoolsd support.
@@ -297,9 +336,36 @@ typedef enum {
     * types (that other plugins can hook into).
     */
    TOOLS_APP_PROVIDER   = 3,
+   /**
+    * Denotes a property made available through the service's instance object
+    * (ToolsAppCtx::serviceObj).
+    */
+   TOOLS_SVC_PROPERTY   = 4,
+
+   /* VMCF applications. */
+
+   /**
+    * Denotes a raw VMCF object handler. See the VMCFObjectProvider struct.
+    */
+   TOOLS_VMCF_OBJECT       = 0x00010001,
+   /**
+    * Denotes a VMCF signal handler. See the VMCFSignalHandler struct.
+    */
+   TOOLS_VMCF_SIGNAL       = 0x00010002,
+   /**
+    * Denotes a VOLT object registration. See the VMCFVoltObject struct.
+    */
+   TOOLS_VMCF_VOLT_OBJECT  = 0x00010003,
+   /**
+    * Denotes a VOLT signal registration. See the VMCFVoltSignal struct.
+    */
+   TOOLS_VMCF_VOLT_SIGNAL  = 0x00010004,
+
 } ToolsAppType;
 
 
+struct ToolsPluginData;
+
 /**
  * Defines the registration data for an "application provider". Application
  * providers allow plugins to hook into new application frameworks that will
@@ -334,18 +400,24 @@ typedef struct ToolsAppProvider {
     * Registration callback. This is called after "activate", to register an
     * application provided by a plugin.
     *
-    * @param[in]  ctx   The application context.
-    * @param[in]  prov  The provider instance.
-    * @param[in]  reg   The application registration data.
+    * @param[in]  ctx      The application context.
+    * @param[in]  prov     The provider instance.
+    * @param[in]  plugin   The plugin that owns the registration.
+    * @param[in]  reg      The application registration data.
     *
     * @return Whether registration succeeded.
     */
-   gboolean (*registerApp)(ToolsAppCtx *ctx, struct ToolsAppProvider *prov, gpointer reg);
+   gboolean (*registerApp)(ToolsAppCtx *ctx,
+                           struct ToolsAppProvider *prov,
+                           struct ToolsPluginData *plugin,
+                           gpointer reg);
    /**
     * Shutdown callback (optional). Called when the service is being shut down.
     * The provider is responsible for keeping track of registrations and
     * cleaning them up during shutdown.
     *
+    * This method is only called if the provider was successfully activated.
+    *
     * @param[in]  ctx   The application context.
     * @param[in]  prov  The provider instance.
     */
@@ -382,6 +454,24 @@ typedef struct ToolsAppReg {
 } ToolsAppReg;
 
 
+/**
+ * Defines a property that is exposed through the containers instance object
+ * (ToolsAppCtx::serviceObj). Plugins can expose properties through this
+ * mechanism to share state with other plugins. Properties can be accessed
+ * using GObject's g_set_property / g_get_property APIs, or monitored by
+ * registering for the "notify" signal on the object.
+ *
+ * All properties exposed using this mechanism are opaque pointers. It's up
+ * for individual plugins to define the actual types of the properties.
+ *
+ * Properties are not ref counted, so consumers (code calling "g_get_property")
+ * should be aware of the life cycle of the property as defined by the producer.
+ */
+typedef struct ToolsServiceProperty {
+   const char    *name;
+} ToolsServiceProperty;
+
+
 /**
  * Defines a struct for mapping callbacks to signals. Normally it would suffice
  * to use g_signal_connect() directly to register interest in signals; but to
@@ -414,7 +504,7 @@ typedef struct ToolsPluginSignalCb {
  */
 typedef struct ToolsPluginData {
    /** Name of the application (required). */
-   char                      *name;
+   char const                *name;
    /**
     * List of features provided by the app. Registration of applications
     * happens in the same order provided by this array.
