@@ -384,7 +384,6 @@ MXUserUp(NativeSemaphore *sema)  // IN:
 #endif  // _WIN32
 
 
-#if defined(MXUSER_STATS)
 /*
  *-----------------------------------------------------------------------------
  *
@@ -442,7 +441,6 @@ MXUserStatsActionSema(MXUserHeader *header)  // IN:
       }
    }
 }
-#endif
 
 
 /*
@@ -471,6 +469,8 @@ MXUserDumpSemaphore(MXUserHeader *header)  // IN:
    Warning("\tsignature 0x%X\n", sema->header.signature);
    Warning("\tname %s\n", sema->header.name);
    Warning("\trank 0x%X\n", sema->header.rank);
+   Warning("\tserial number %u\n", sema->header.serialNumber);
+
    Warning("\treference count %u\n", Atomic_Read(&sema->activeUserCount));
    Warning("\tnative semaphore 0x%p\n", &sema->nativeSemaphore);
 }
@@ -514,23 +514,24 @@ MXUser_CreateSemaphore(const char *userName,  // IN:
    if (LIKELY(MXUserInit(&sema->nativeSemaphore) == 0)) {
       MXUserStats *stats;
 
-      sema->header.name = properName;
       sema->header.signature = MXUSER_SEMA_SIGNATURE;
+      sema->header.name = properName;
       sema->header.rank = rank;
+      sema->header.serialNumber = MXUserAllocSerialNumber();
       sema->header.dumpFunc = MXUserDumpSemaphore;
 
-#if defined(MXUSER_STATS)
-      sema->header.statsFunc = MXUserStatsActionSema;
-      sema->header.identifier = MXUserAllocID();
+      if (MXUserStatsEnabled()) {
+         sema->header.statsFunc = MXUserStatsActionSema;
 
-      stats = Util_SafeCalloc(1, sizeof(*stats));
+         stats = Util_SafeCalloc(1, sizeof(*stats));
 
-      MXUserAcquisitionStatsSetUp(&stats->acquisitionStats);
-#else
-      stats = NULL;
-#endif
+         MXUserAcquisitionStatsSetUp(&stats->acquisitionStats);
 
-      Atomic_WritePtr(&sema->statsMem, stats);
+         Atomic_WritePtr(&sema->statsMem, stats);
+      } else {
+         sema->header.statsFunc = NULL;
+         Atomic_WritePtr(&sema->statsMem, NULL);
+      }
 
       MXUserAddToList(&sema->header);
    } else {
@@ -593,7 +594,7 @@ MXUser_DestroySemaphore(MXUserSemaphore *sema)  // IN:
       }
 
       sema->header.signature = 0;  // just in case...
-      free((void *) sema->header.name);  // avoid const warnings
+      free(sema->header.name);
       sema->header.name = NULL;
       free(sema);
    }
@@ -653,7 +654,7 @@ MXUser_DownSemaphore(MXUserSemaphore *sema)  // IN/OUT:
             histo = Atomic_ReadPtr(&stats->acquisitionHisto);
 
             if (UNLIKELY(histo != NULL)) {
-               MXUserHistoSample(histo, value);
+               MXUserHistoSample(histo, value, GetReturnAddress());
             }
          }
       }
@@ -731,7 +732,7 @@ MXUser_TimedDownSemaphore(MXUserSemaphore *sema,  // IN/OUT:
                MXUserHisto *histo = Atomic_ReadPtr(&stats->acquisitionHisto);
 
                if (UNLIKELY(histo != NULL)) {
-                  MXUserHistoSample(histo, value);
+                  MXUserHistoSample(histo, value, GetReturnAddress());
                }
             }
          }
