@@ -27,6 +27,7 @@
 #include <fcntl.h>
 #include <limits.h>
 #include <errno.h>
+#include <sys/file.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <sys/wait.h>
@@ -399,8 +400,6 @@ Hostinfo_GetSystemBitness(void)
    } else {
       return 32;
    }
-#elif defined N_PLAT_NLM
-   return 32;
 #else
    char buf[SYSTEM_BITNESS_MAXLEN] = { '\0', };
 #   if defined __FreeBSD__ || defined __APPLE__
@@ -559,6 +558,8 @@ HostinfoGetOSShortName(char *distro,         // IN: full distro name
          Str_Strcpy(distroShort, STR_OS_DEBIAN_4, distroShortSize);
       } else if (strstr(distroLower, "5.0")) {
          Str_Strcpy(distroShort, STR_OS_DEBIAN_5, distroShortSize);
+      } else if (strstr(distroLower, "6.0")) {
+         Str_Strcpy(distroShort, STR_OS_DEBIAN_6, distroShortSize);
       }
    } else if (StrUtil_StartsWith(distroLower, "enterprise linux") ||
               StrUtil_StartsWith(distroLower, "oracle")) {
@@ -631,7 +632,7 @@ HostinfoReadDistroFile(char *filename,  // IN: distro version file name
                        int distroSize,  // IN: size of OS distro name buffer
                        char *distro)    // OUT: full distro name
 {
-   int fd;
+   int fd = -1;
    int buf_sz;
    struct stat st;
    Bool ret = FALSE;
@@ -640,10 +641,8 @@ HostinfoReadDistroFile(char *filename,  // IN: distro version file name
    char *tmpDistroPos = NULL;
    int i = 0;
 
-   if ((fd = open(filename, O_RDONLY)) < 0) {
-      Warning("%s: could not open file%s: %d\n", __FUNCTION__, filename,
-              errno);
-
+   /* It's OK for the file to not exist, don't warn for this.  */
+   if ((fd = Posix_Open(filename, O_RDONLY)) == -1) {
       return FALSE;
    }
 
@@ -667,9 +666,7 @@ HostinfoReadDistroFile(char *filename,  // IN: distro version file name
 
    if (distroOrig == NULL) {
       Warning("%s: could not allocate memory\n", __FUNCTION__);
-      close(fd);
-
-      return FALSE;
+      goto out;
    }
 
    if (read(fd, distroOrig, buf_sz) != buf_sz) {
@@ -716,7 +713,9 @@ HostinfoReadDistroFile(char *filename,  // IN: distro version file name
    ret = TRUE;
 
 out:
-   close(fd);
+   if (fd != -1) {
+      close(fd);
+   }
    free(distroOrig);
 
    return ret;
@@ -743,10 +742,6 @@ out:
 static char *
 HostinfoGetCmdOutput(const char *cmd)  // IN:
 {
-#if defined(N_PLAT_NLM)
-   Warning("Trying to execute command \"%s\" and catch its output... No way on NetWare...\n", cmd);
-   return NULL;
-#else
    DynBuf db;
    FILE *stream;
    char *out = NULL;
@@ -797,7 +792,6 @@ HostinfoGetCmdOutput(const char *cmd)  // IN:
    pclose(stream);
 
    return out;
-#endif
 }
 
 
@@ -862,7 +856,6 @@ HostinfoOSData(void)
        * drive as a side-effect. So use "system_profiler SPSoftwareDataType"
        * instead.
        */
-      SInt32 major;
       char *sysname = HostinfoGetCmdOutput(
                          "/usr/sbin/system_profiler SPSoftwareDataType"
                       " | /usr/bin/grep 'System Version:'"
@@ -880,22 +873,8 @@ HostinfoOSData(void)
          /* Fall back to returning the original osNameFull. */
       }
 
-      if (Gestalt(gestaltSystemVersionMajor, &major) == 0) {
-         /*
-          * XXX: some places in the code refer to Mac OS X Lion (10.7?)
-          * as "darwin11", which is not what this code is doing. Once it's
-          * released and we support it, this code may need some tweaking.
-          */
-         Str_Snprintf(osName, sizeof osName, "%s%d", STR_OS_MACOS, (int) major);
-      } else if (Gestalt(gestaltSystemVersion, &major) == 0) {
-         /* Pre 10.3 Mac OS doesn't have gestaltSystemVersionMajor. */
-         NOT_TESTED();
-         major = (major & 0xFF00) >> 8;
-         Str_Snprintf(osName, sizeof osName, "%s%x", STR_OS_MACOS, (int) major);
-      } else {
-         Log("Call to Gestalt() failed!\n");
-         Str_Snprintf(osName, sizeof osName, "%s", STR_OS_MACOS);
-      }
+      Str_Snprintf(osName, sizeof osName, "%s%d", STR_OS_MACOS,
+                   Hostinfo_OSVersion(0));
    }
 #else
    // XXX Use compile-time instead of run-time checks for these as well.
@@ -904,21 +883,23 @@ HostinfoOSData(void)
       char distroShort[DISTRO_BUF_SIZE];
       static int const distroSize = sizeof distro;
       char *lsbOutput;
+      int majorVersion;
 
       /*
        * Write default distro string depending on the kernel version. If
        * later we find more detailed information this will get overwritten.
        */
 
-      if (StrUtil_StartsWith(buf.release, "2.4.")) {
-         Str_Strcpy(distro, STR_OS_OTHER_24_FULL, distroSize);
-         Str_Strcpy(distroShort, STR_OS_OTHER_24, distroSize);
-      } else if (StrUtil_StartsWith(buf.release, "2.6.")) {
-         Str_Strcpy(distro, STR_OS_OTHER_26_FULL, distroSize);
-         Str_Strcpy(distroShort, STR_OS_OTHER_26, distroSize);
-      } else {
+      majorVersion = Hostinfo_OSVersion(0);
+      if (majorVersion < 2 || (majorVersion == 2 && Hostinfo_OSVersion(1) < 4)) {
          Str_Strcpy(distro, STR_OS_OTHER_FULL, distroSize);
          Str_Strcpy(distroShort, STR_OS_OTHER, distroSize);
+      } else if (majorVersion == 2 && Hostinfo_OSVersion(1) < 6) {
+         Str_Strcpy(distro, STR_OS_OTHER_24_FULL, distroSize);
+         Str_Strcpy(distroShort, STR_OS_OTHER_24, distroSize);
+      } else {
+         Str_Strcpy(distro, STR_OS_OTHER_26_FULL, distroSize);
+         Str_Strcpy(distroShort, STR_OS_OTHER_26, distroSize);
       }
 
       /*
@@ -1741,6 +1722,9 @@ HostinfoSystemTimerPosix(VmTimeType *result)  // OUT
  *      The actual resolution of these "clocks" are undefined - it varies
  *      depending on hardware, OSen and OS versions.
  *
+ *     *** NOTE: This function and all children must be callable
+ *     while RANK_logLock is held. ***
+ *
  * Results:
  *      The time in nanoseconds is returned. Zero upon error.
  *
@@ -1935,6 +1919,10 @@ Hostinfo_ResetProcessState(const int *keepFds, // IN:
  *      (as a US-ASCII string followed by a newline) of the daemon
  *      process to that path.
  *
+ *      If 'flags' contains HOSTINFO_DAEMONIZE_LOCKPID and pidPath is
+ *      non-NULL, then an exclusive flock(2) is taken on pidPath to prevent
+ *      multiple instances of the service from running.
+ *
  * Results:
  *      FALSE if the process could not be daemonized.  errno contains
  *      the error on failure.
@@ -1978,13 +1966,14 @@ Hostinfo_Daemonize(const char *path,             // IN: NUL-terminated UTF-8
     * causes them to reopen their Mach ports.
     */
 
+   int pidPathFd = -1;
    int childPid;
    int pipeFds[2] = { -1, -1 };
    uint32 err = EINVAL;
    char *pathLocalEncoding = NULL;
-   char *pidPathLocalEncoding = NULL;
    char **argsLocalEncoding = NULL;
    int *tempFds = NULL;
+   size_t numTempFds = numKeepFds + 1;
    sigset_t sig;
 
    ASSERT_ON_COMPILE(sizeof (errno) <= sizeof err);
@@ -1992,6 +1981,39 @@ Hostinfo_Daemonize(const char *path,             // IN: NUL-terminated UTF-8
    ASSERT(path);
    ASSERT(numKeepFds == 0 || keepFds);
 
+   if (pidPath) {
+      pidPathFd = Posix_Open(pidPath, O_WRONLY | O_CREAT, 0644);
+      if (pidPathFd == -1) {
+         err = errno;
+         Warning("%s: Couldn't open PID path [%s], error %u.\n",
+                 __FUNCTION__, pidPath, err);
+         errno = err;
+         return FALSE;
+      }
+
+      /*
+       * Lock this file to take a mutex on daemonizing this process. The child
+       * will keep this file descriptor open for as long as it is running.
+       *
+       * flock(2) is a BSD extension (also supported on Linux) which creates a
+       * lock that is inherited by the child after fork(2). fcntl(2) locks do
+       * not have this property. Solaris only supports fcntl(2) locks.
+       */
+#ifndef sun
+      if ((flags & HOSTINFO_DAEMONIZE_LOCKPID) &&
+          flock(pidPathFd, LOCK_EX | LOCK_NB) == -1) {
+         err = errno;
+         Warning("%s: Lock held on PID path [%s], error %u, not daemonizing.\n",
+                 __FUNCTION__, pidPath, err);
+         errno = err;
+         close(pidPathFd);
+         return FALSE;
+      }
+#endif
+
+      numTempFds++;
+   }
+
    if (pipe(pipeFds) == -1) {
       err = errno;
       Warning("%s: Couldn't create pipe, error %u.\n", __FUNCTION__, err);
@@ -1999,16 +2021,18 @@ Hostinfo_Daemonize(const char *path,             // IN: NUL-terminated UTF-8
       goto cleanup;
    }
 
-   numKeepFds++;
-   tempFds = malloc(sizeof tempFds[0] * numKeepFds);
+   tempFds = malloc(sizeof tempFds[0] * numTempFds);
    if (!tempFds) {
       err = errno;
       Warning("%s: Couldn't allocate memory, error %u.\n", __FUNCTION__, err);
       goto cleanup;
    }
-   tempFds[0] = pipeFds[1];
    if (keepFds) {
-      memcpy(tempFds + 1, keepFds, sizeof tempFds[0] * (numKeepFds - 1));
+      memcpy(tempFds, keepFds, sizeof tempFds[0] * numKeepFds);
+   }
+   tempFds[numKeepFds++] = pipeFds[1];
+   if (pidPath) {
+      tempFds[numKeepFds++] = pidPathFd;
    }
 
    if (fcntl(pipeFds[1], F_SETFD, 1) == -1) {
@@ -2026,17 +2050,6 @@ Hostinfo_Daemonize(const char *path,             // IN: NUL-terminated UTF-8
       goto cleanup;
    }
 
-   if (pidPath) {
-      pidPathLocalEncoding = Unicode_GetAllocBytes(pidPath,
-                                                   STRING_ENCODING_DEFAULT);
-
-      if (!pidPathLocalEncoding) {
-         Warning("%s: Couldn't convert path [%s] to default encoding.\n",
-                 __FUNCTION__, pidPath);
-         goto cleanup;
-      }
-   }
-
    argsLocalEncoding = Unicode_GetAllocList(args, STRING_ENCODING_DEFAULT, -1);
    if (!argsLocalEncoding) {
       Warning("%s: Couldn't convert arguments to default encoding.\n",
@@ -2101,9 +2114,9 @@ Hostinfo_Daemonize(const char *path,             // IN: NUL-terminated UTF-8
 
    /*
     * Close all fds except for the write end of the error pipe (which we've
-    * already set to close on successful exec), and the ones requested by
-    * the caller. Also reset the signal mask to unblock all signals. fork()
-    * clears pending signals.
+    * already set to close on successful exec), the pid file, and the ones
+    * requested by the caller. Also reset the signal mask to unblock all
+    * signals. fork() clears pending signals.
     */
 
    Hostinfo_ResetProcessState(tempFds, numKeepFds);
@@ -2182,18 +2195,15 @@ Hostinfo_Daemonize(const char *path,             // IN: NUL-terminated UTF-8
       int64 pid;
       char pidString[32];
       int pidStringLen;
-      int pidPathFd;
 
       ASSERT_ON_COMPILE(sizeof (pid_t) <= sizeof pid);
-      ASSERT(pidPathLocalEncoding);
+      ASSERT(pidPathFd >= 0);
 
-      /* See above comment about how we can't use our i18n wrappers here. */
-      pidPathFd = open(pidPathLocalEncoding, O_WRONLY|O_CREAT|O_TRUNC, 0644);
-
-      if (pidPathFd == -1) {
-         err = errno;
-         Warning("%s: Couldn't open PID path [%s], error %d.\n",
-                 __FUNCTION__, pidPath, err);
+      pid = getpid();
+      pidStringLen = Str_Sprintf(pidString, sizeof pidString,
+                                 "%"FMT64"d\n", pid);
+      if (pidStringLen <= 0) {
+         err = EINVAL;
 
          if (write(pipeFds[1], &err, sizeof err) == -1) {
             Warning("%s: Couldn't write to parent pipe: %u, original "
@@ -2202,11 +2212,10 @@ Hostinfo_Daemonize(const char *path,             // IN: NUL-terminated UTF-8
          _exit(EXIT_FAILURE);
       }
 
-      pid = getpid();
-      pidStringLen = Str_Sprintf(pidString, sizeof pidString,
-                                 "%"FMT64"d\n", pid);
-      if (pidStringLen <= 0) {
-         err = EINVAL;
+      if (ftruncate(pidPathFd, 0) == -1) {
+         err = errno;
+         Warning("%s: Couldn't truncate path [%s], error %d.\n",
+                 __FUNCTION__, pidPath, err);
 
          if (write(pipeFds[1], &err, sizeof err) == -1) {
             Warning("%s: Couldn't write to parent pipe: %u, original "
@@ -2227,7 +2236,22 @@ Hostinfo_Daemonize(const char *path,             // IN: NUL-terminated UTF-8
          _exit(EXIT_FAILURE);
       }
 
-      close(pidPathFd);
+      if (fsync(pidPathFd) == -1) {
+         err = errno;
+         Warning("%s: Couldn't flush PID to path [%s], error %d.\n",
+                 __FUNCTION__, pidPath, err);
+
+         if (write(pipeFds[1], &err, sizeof err) == -1) {
+            Warning("%s: Couldn't write to parent pipe: %u, original "
+                    "error: %u.\n", __FUNCTION__, errno, err);
+         }
+         _exit(EXIT_FAILURE);
+      }
+
+      /* Leave pidPathFd open to hold the mutex until this process exits. */
+      if (!(flags & HOSTINFO_DAEMONIZE_LOCKPID)) {
+         close(pidPathFd);
+      }
    }
 
    if (execv(pathLocalEncoding, argsLocalEncoding) == -1) {
@@ -2254,7 +2278,6 @@ Hostinfo_Daemonize(const char *path,             // IN: NUL-terminated UTF-8
       close(pipeFds[1]);
    }
    Util_FreeStringList(argsLocalEncoding, -1);
-   free(pidPathLocalEncoding);
    free(pathLocalEncoding);
 
    if (err == 0) {
@@ -2263,12 +2286,21 @@ Hostinfo_Daemonize(const char *path,             // IN: NUL-terminated UTF-8
       }
    } else {
       if (pidPath) {
+         /*
+          * Unlink pidPath on error before closing pidPathFd to avoid racing
+          * with another process attempting to daemonize and unlinking the
+          * file it created instead.
+          */
          Posix_Unlink(pidPath);
       }
 
       errno = err;
    }
 
+   if (pidPath) {
+      close(pidPathFd);
+   }
+
    return err == 0;
 }
 
@@ -2479,11 +2511,11 @@ Hostinfo_GetCpuDescription(uint32 cpuNumber)  // IN:
  *
  * Hostinfo_Execute --
  *
- *      Start program COMMAND.  If WAIT is TRUE, wait for program
+ *      Start program 'path'.  If 'wait' is TRUE, wait for program
  *	to complete and return exit status.
  *
  * Results:
- *      Exit status of COMMAND.
+ *      Exit status of 'path'.
  *
  * Side effects:
  *      Run a separate program.
@@ -2492,14 +2524,16 @@ Hostinfo_GetCpuDescription(uint32 cpuNumber)  // IN:
  */
 
 int
-Hostinfo_Execute(const char *command,  // IN:
-		 char * const *args,   // IN:
-		 Bool wait)            // IN:
+Hostinfo_Execute(const char *path,   // IN:
+                 char * const *args, // IN:
+                 Bool wait,          // IN:
+                 const int *keepFds, // IN/OPT: array of fds to be kept open
+                 size_t numKeepFds)  // IN: number of fds in keepFds
 {
    int pid;
    int status;
 
-   if (command == NULL) {
+   if (path == NULL) {
       return 1;
    }
 
@@ -2510,8 +2544,8 @@ Hostinfo_Execute(const char *command,  // IN:
    }
 
    if (pid == 0) {
-      Hostinfo_ResetProcessState(NULL, 0);
-      Posix_Execvp(command, args);
+      Hostinfo_ResetProcessState(keepFds, numKeepFds);
+      Posix_Execvp(path, args);
       exit(127);
    }
 
@@ -3252,7 +3286,8 @@ Hostinfo_GetModulePath(uint32 priv)  // IN:
 #endif
 
    // "/proc/self/exe" only exists on Linux 2.2+.
-   ASSERT(Hostinfo_OSVersion(0) >= 2 && Hostinfo_OSVersion(1) >= 2);
+   ASSERT(Hostinfo_OSVersion(0) > 2 ||
+          (Hostinfo_OSVersion(0) == 2 && Hostinfo_OSVersion(1) >= 2));
 
    if (priv == HGMP_PRIVILEGE) {
       uid = Id_BeginSuperUser();
