@@ -99,7 +99,8 @@ struct ProcMgr_AsyncProc {
 };
 
 static pid_t ProcMgrStartProcess(char const *cmd,
-                                 char * const  *envp);
+                                 char * const  *envp,
+                                 char const *workingDir);
 
 static Bool ProcMgrWaitForProcCompletion(pid_t pid,
                                          Bool *validExitCode,
@@ -109,6 +110,10 @@ static Bool ProcMgrKill(pid_t pid,
                         int sig,
                         int timeout);
 
+#ifdef sun
+#define  SOLARIS_BASH_PATH "/usr/bin/bash"
+#endif
+
 #if defined(linux) && !defined(GLIBC_VERSION_23)
 /*
  * Implements the system calls (they are not wrapped by glibc til 2.3.2).
@@ -138,6 +143,92 @@ setresgid(gid_t ruid,
 #endif
 
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * ProcMgr_ReadProcFile --
+ *
+ *    Read the contents of a file in /proc/<pid>.
+ *
+ *    The size is essentially unbounded because of cmdline arguments.
+ *    The only way to figure out the content size is to keep reading;
+ *    stat(2) and lseek(2) lie.
+ *
+ *    The contents are NUL terminated -- in some distros may not include
+ *    a NUL for some commands (eg. pid 1, /sbin/init) -- so add
+ *    one to be safe.
+ *
+ * Results:
+ *
+ *    The length of the file.
+ *
+ *    -1 on error.
+ *
+ * Side effects:
+ *
+ *    The returned contents must be freed by caller.
+ *
+ *----------------------------------------------------------------------
+ */
+
+#if !defined(sun)
+int
+ProcMgr_ReadProcFile(int fd,                       // IN
+                     char **contents)              // OUT
+{
+   int size = 0;
+#if !defined(__FreeBSD__) && !defined(__APPLE__)
+   char tmp[512];
+   int numRead;
+
+   *contents = NULL;
+   numRead = read(fd, tmp, sizeof(tmp));
+   size = numRead;
+
+   if (numRead <= 0) {
+      goto done;
+   }
+
+   /*
+    * handle the 99% case
+    */
+   if (sizeof(tmp) > numRead) {
+      char *result;
+
+      result = malloc(numRead + 1);
+      if (NULL == result) {
+         size = -1;
+         goto done;
+      }
+      memcpy(result, tmp, numRead);
+      result[numRead] = '\0';
+      *contents = result;
+      goto done;
+   } else {
+      DynBuf dbuf;
+
+      DynBuf_Init(&dbuf);
+      DynBuf_Append(&dbuf, tmp, numRead);
+      do {
+         numRead = read(fd, tmp, sizeof(tmp));
+         if (numRead > 0) {
+            DynBuf_Append(&dbuf, tmp, numRead);
+         }
+         size += numRead;
+      } while (numRead > 0);
+      // add the NUL term
+      DynBuf_Append(&dbuf, "", 1);
+      DynBuf_Trim(&dbuf);
+      *contents = DynBuf_Detach(&dbuf);
+      DynBuf_Destroy(&dbuf);
+   }
+done:
+#endif
+   return size;
+}
+
+#endif   // !sun
+
 /*
  *----------------------------------------------------------------------
  *
@@ -156,11 +247,12 @@ setresgid(gid_t ruid,
  *----------------------------------------------------------------------
  */
 
+#if !defined(sun)
 ProcMgr_ProcList *
 ProcMgr_ListProcesses(void)
 {
    ProcMgr_ProcList *procList = NULL;
-#if !defined(__FreeBSD__) && !defined(sun) && !defined(__APPLE__)
+#if !defined(__FreeBSD__) && !defined(__APPLE__)
    Bool failed = FALSE;
    DynBuf dbProcId;
    DynBuf dbProcCmd;
@@ -244,8 +336,8 @@ ProcMgr_ListProcesses(void)
       pid_t pid;
       int replaceLoop;
       struct passwd *pwd;
-      char cmdLineTemp[2048];
-      char cmdStatTemp[2048];
+      char *cmdLineTemp = NULL;
+      char *cmdStatTemp = NULL;
       char *cmdLine;
       char *userName = NULL;
       size_t strLen = 0;
@@ -290,14 +382,18 @@ ProcMgr_ListProcesses(void)
        * In the future, we could keep the NUL version around and pass it
        * back to the client for easier parsing when retrieving individual
        * command line parameters is needed.
-       *
-       * We read at most (sizeof cmdLineTemp) - 1 bytes to leave room
-       * for NUL termination at the end.
        */
-      numRead = read(cmdFd, cmdLineTemp, sizeof cmdLineTemp - sizeof(char));
+      numRead = ProcMgr_ReadProcFile(cmdFd, &cmdLineTemp);
       close(cmdFd);
 
+      if (numRead < 0) {
+         continue;
+      }
+
       if (numRead > 0) {
+         /*
+          * Stop before we hit the final '\0'; want to leave it alone.
+          */
          for (replaceLoop = 0 ; replaceLoop < (numRead - 1) ; replaceLoop++) {
             if ('\0' == cmdLineTemp[replaceLoop]) {
                cmdLineTemp[replaceLoop] = ' ';
@@ -320,17 +416,11 @@ ProcMgr_ListProcesses(void)
             cmdFd = open(cmdFilePath, O_RDONLY);
          }
          if (cmdFd != -1) {
-            numRead = read(cmdFd, cmdLineTemp, sizeof(cmdLineTemp) - sizeof(char));
+            numRead = ProcMgr_ReadProcFile(cmdFd, &cmdLineTemp);
             close(cmdFd);
-
-            if (numRead < 0) {
-               cmdLineTemp[0] = '\0';
-            } else {
-               cmdLineTemp[numRead] = '\0';
-            }
          }
          if (numRead > 0) {
-            /* 
+            /*
              * Extract the part with just the name, by reading until the first
              * space, then reading the next non-space word after that, and
              * ignoring everything else. The format looks like this:
@@ -338,8 +428,8 @@ ProcMgr_ListProcesses(void)
              * for example:
              *     "Name:    nfsd"
              */
-            const char* nameStart = NULL;
-            char* copyItr = NULL;
+            const char *nameStart;
+            char *copyItr;
 
             /* Skip non-whitespace. */
             for (nameStart = cmdLineTemp; *nameStart && 
@@ -359,21 +449,6 @@ ProcMgr_ListProcesses(void)
          }
       }
 
-      /*
-       * There is an edge case where /proc/#/cmdline does not NUL terminate
-       * the command.  /sbin/init (process 1) is like that on some distros.
-       * So let's guarantee that the string is NUL terminated, even if
-       * the last character of the string might already be NUL.
-       * This is safe to do because we read at most (sizeof cmdLineTemp) - 1
-       * bytes from /proc/#/cmdline -- we left just enough space to add
-       * NUL termination at the end.
-       */
-      if (numRead < 0) {
-         cmdLineTemp[0] = '\0';
-      } else {
-         cmdLineTemp[numRead] = '\0';
-      }
-
       /*
        * Get the inode information for this process.  This gives us
        * the process owner.
@@ -383,7 +458,7 @@ ProcMgr_ListProcesses(void)
                    "/proc/%s",
                    ent->d_name) == -1) {
          Debug("Giant process id '%s'\n", ent->d_name);
-         continue;
+         goto next_entry;
       }
 
       /*
@@ -393,7 +468,7 @@ ProcMgr_ListProcesses(void)
        */
       statResult = stat(cmdFilePath, &fileStat);
       if (0 != statResult) {
-         continue;
+         goto next_entry;
       }
 
       /*
@@ -405,22 +480,22 @@ ProcMgr_ListProcesses(void)
                    "/proc/%s/stat",
                    ent->d_name) == -1) {
          Debug("Giant process id '%s'\n", ent->d_name);
-         continue;
+         goto next_entry;
       }
       cmdFd = open(cmdFilePath, O_RDONLY);
       if (-1 == cmdFd) {
-         continue;
+         goto next_entry;
       }
-      numRead = read(cmdFd, cmdStatTemp, sizeof cmdStatTemp);
+      numRead = ProcMgr_ReadProcFile(cmdFd, &cmdStatTemp);
       close(cmdFd);
       if (0 >= numRead) {
-         continue;
+         goto next_entry;
       }
       /*
        * Skip over initial process id and process name.  "123 (bash) [...]".
        */
       stringBegin = strchr(cmdStatTemp, ')') + 2;
-      
+
       numberFound = sscanf(stringBegin, "%c %d %d %d %d %d "
                            "%lu %lu %lu %lu %lu %Lu %Lu %Lu %Lu %ld %ld "
                            "%d %ld %Lu",
@@ -437,14 +512,18 @@ ProcMgr_ListProcesses(void)
                            (int *) &dummy, (long *) &dummy,
                            &relativeStartTime);
       if (20 != numberFound) {
-         continue;
+         goto next_entry;
       }
       processStartTime = hostStartTime + (relativeStartTime / hertz);
 
       /*
        * Store the command line string pointer in dynbuf.
        */
-      cmdLine = strdup(cmdLineTemp);
+      if (cmdLineTemp) {
+         cmdLine = Unicode_Alloc(cmdLineTemp, STRING_ENCODING_DEFAULT);
+      } else {
+         cmdLine = Unicode_Alloc("", STRING_ENCODING_UTF8);
+      }
       DynBuf_Append(&dbProcCmd, &cmdLine, sizeof cmdLine);
 
       /*
@@ -459,7 +538,7 @@ ProcMgr_ListProcesses(void)
       pwd = getpwuid(fileStat.st_uid);
       userName = (NULL == pwd)
                  ? Str_Asprintf(&strLen, "%d", (int) fileStat.st_uid)
-                 : Util_SafeStrdup(pwd->pw_name);
+                 : Unicode_Alloc(pwd->pw_name, STRING_ENCODING_DEFAULT);
       DynBuf_Append(&dbProcOwner, &userName, sizeof userName);
 
       /*
@@ -468,6 +547,10 @@ ProcMgr_ListProcesses(void)
       DynBuf_Append(&dbProcStartTime,
                     &processStartTime,
                     sizeof processStartTime);
+
+next_entry:
+      free(cmdLineTemp);
+      free(cmdStatTemp);
    } // while readdir
 
    closedir(dir);
@@ -512,10 +595,11 @@ abort:
       ProcMgr_FreeProcList(procList);
       procList = NULL;
    }
-#endif // !defined(__FreeBSD__) && !defined(sun) && !defined(__APPLE__)
+#endif // !defined(__FreeBSD__) && !defined(__APPLE__)
 
    return procList;
 }
+#endif // !defined(sun)
 
 
 /*
@@ -581,7 +665,8 @@ ProcMgr_ExecSync(char const *cmd,                  // IN: UTF-8 command line
 
    Debug("Executing sync command: %s\n", cmd);
 
-   pid = ProcMgrStartProcess(cmd, userArgs ? userArgs->envp : NULL);
+   pid = ProcMgrStartProcess(cmd, userArgs ? userArgs->envp : NULL,
+                             userArgs ? userArgs->workingDirectory : NULL);
 
    if (pid == -1) {
       return FALSE;
@@ -610,11 +695,13 @@ ProcMgr_ExecSync(char const *cmd,                  // IN: UTF-8 command line
 
 static pid_t
 ProcMgrStartProcess(char const *cmd,            // IN: UTF-8 encoded cmd
-                    char * const *envp)         // IN: UTF-8 encoded env vars
+                    char * const *envp,         // IN: UTF-8 encoded env vars
+                    char const *workingDir)     // IN: UTF-8 working directory
 {
    pid_t pid;
    char *cmdCurrent = NULL;
    char **envpCurrent = NULL;
+   char *workDir = NULL;
 
    if (cmd == NULL) {
       ASSERT(FALSE);
@@ -631,6 +718,12 @@ ProcMgrStartProcess(char const *cmd,            // IN: UTF-8 encoded cmd
       return -1;
    }
 
+   if ((NULL != workingDir) &&
+       !CodeSet_Utf8ToCurrent(workingDir, strlen(workingDir), &workDir, NULL)) {
+      Warning("Could not convert workingDir from UTF-8 to current\n");
+      return -1;
+   }
+
    if (NULL != envp) {
       envpCurrent = Unicode_GetAllocList(envp, -1, STRING_ENCODING_DEFAULT);
    }
@@ -640,13 +733,52 @@ ProcMgrStartProcess(char const *cmd,            // IN: UTF-8 encoded cmd
    if (pid == -1) {
       Warning("Unable to fork: %s.\n\n", strerror(errno));
    } else if (pid == 0) {
+#ifdef sun
+      /*
+       * On Solaris, /bin/sh is the Bourne shell, and it
+       * doesn't appear to have the optimization that bash does -- when
+       * called with -c, bash appears to just use exec() to replace itself.
+       * Bourne shell does a fork & exec, so 2 processes are started.
+       * This is bad for us because we then see the PID of the shell, not the
+       * app that it starts.  When this PID is returned to a user to
+       * watch, they'll watch the wrong process.
+       *
+       * So for Solaris, use bash instead if possible.  We support
+       * Solaris 10 and better; it contains bash, but not in its
+       * minimal 'core' package, so it may not exist.
+       */
+      static const char bashShellPath[] = SOLARIS_BASH_PATH;
+      char *bashArgs[] = { "bash", "-c", cmdCurrent, NULL };
+      static const char bourneShellPath[] = "/bin/sh";
+      char *bourneArgs[] = { "sh", "-c", cmdCurrent, NULL };
+      const char *shellPath;
+      char **args;
+#else
       static const char shellPath[] = "/bin/sh";
       char *args[] = { "sh", "-c", cmdCurrent, NULL };
+#endif
+
+#ifdef sun
+      if (File_Exists(SOLARIS_BASH_PATH)) {
+         shellPath = bashShellPath;
+         args = bashArgs;
+      } else {
+         shellPath = bourneShellPath;
+         args = bourneArgs;
+      }
+#endif
 
       /*
        * Child
        */
 
+      if (NULL != workDir) {
+         if (chdir(workDir) != 0) {
+            Warning("%s: Could not chdir(%s) %s\n", __FUNCTION__, workDir,
+                    strerror(errno));
+         }
+      }
+
       if (NULL != envpCurrent) {
          execve(shellPath, args, envpCurrent);
       } else  {
@@ -663,6 +795,7 @@ ProcMgrStartProcess(char const *cmd,            // IN: UTF-8 encoded cmd
     */
 
    free(cmdCurrent);
+   free(workDir);
    Unicode_FreeList(envpCurrent, -1);
    return pid;
 }
@@ -763,7 +896,8 @@ ProcMgr_ExecAsync(char const *cmd,                 // IN: UTF-8 command line
    FileIODescriptor readFd;
    FileIODescriptor writeFd;
 
-   Debug("Executing async command: %s\n", cmd);
+   Debug("Executing async command: '%s' in working dir '%s'\n",
+         cmd, (userArgs && userArgs->workingDirectory) ? userArgs->workingDirectory : "");
    
    if (pipe(fds) == -1) {
       Warning("Unable to create the pipe to launch command: %s.\n", cmd);
@@ -819,7 +953,9 @@ ProcMgr_ExecAsync(char const *cmd,                 // IN: UTF-8 command line
        * Only run the program if we have not already experienced a failure.
        */
       if (status) {
-         childPid = ProcMgrStartProcess(cmd, userArgs ? userArgs->envp : NULL);
+         childPid = ProcMgrStartProcess(cmd,
+                                        userArgs ? userArgs->envp : NULL,
+                                        userArgs ? userArgs->workingDirectory : NULL);
          status = childPid != -1;
       }
 
@@ -1452,4 +1588,50 @@ ProcMgr_ImpersonateUserStop(void)
    return TRUE;
 }
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * ProcMgr_GetImpersonatedUserInfo --
+ *
+ *      Return info about the impersonated user.
+ *
+ * Results:
+ *      TRUE on success
+ *
+ * Side effects:
+ *
+ *----------------------------------------------------------------------
+ */
+
+Bool
+ProcMgr_GetImpersonatedUserInfo(char **userName,            // OUT
+                                char **homeDir)             // OUT
+{
+   uid_t uid = geteuid();
+   char buffer[BUFSIZ];
+   struct passwd pw;
+   struct passwd *ppw = &pw;
+   int error;
+
+   *userName = NULL;
+   *homeDir = NULL;
+   if ((error = getpwuid_r(uid, &pw, buffer, sizeof buffer, &ppw)) != 0 ||
+       !ppw) {
+      /*
+       * getpwuid_r() and getpwnam_r() can return a 0 (success) but not
+       * set the return pointer (ppw) if there's no entry for the user,
+       * according to POSIX 1003.1-2003, so patch up the errno.
+       */
+      if (error == 0) {
+         error = ENOENT;
+      }
+      return FALSE;
+   }
+
+   *userName = Unicode_Alloc(ppw->pw_name, STRING_ENCODING_DEFAULT);
+   *homeDir = Unicode_Alloc(ppw->pw_dir, STRING_ENCODING_DEFAULT);
+
+   return TRUE;
+}
+
 #endif // linux
