@@ -190,23 +190,6 @@ typedef int (*VMCIEventReleaseCB)(void *clientData);
 #define VMCI_LOCK_RANK_LOW         (VMCI_LOCK_RANK_MIDDLE_LOW-1)
 #define VMCI_LOCK_RANK_LOWEST      (VMCI_LOCK_RANK_LOW-1)
 
-
-/*
- * In vmkernel, we try to reduce the amount of memory mapped into the
- * virtual address space by only mapping the memory of buffered
- * datagrams when copying from and to the guest. In other OSes,
- * regular kernel memory is used. VMCIBuffer is used to reference
- * possibly unmapped memory.
- */
-
-#ifdef VMKERNEL
-typedef MPN VMCIBuffer;
-#define VMCI_BUFFER_INVALID INVALID_MPN
-#else
-typedef void * VMCIBuffer;
-#define VMCI_BUFFER_INVALID NULL
-#endif
-
 /*
  * Host specific struct used for signalling.
  */
@@ -255,7 +238,7 @@ void VMCI_ReadPortBytes(VMCIIoHandle handle, VMCIIoPort port, uint8 *buffer,
                         size_t bufferLength);
 
 
-void VMCI_InitLock(VMCILock *lock, char *name, VMCILockRank rank);
+int VMCI_InitLock(VMCILock *lock, char *name, VMCILockRank rank);
 void VMCI_CleanupLock(VMCILock *lock);
 void VMCI_GrabLock(VMCILock *lock, VMCILockFlags *flags);
 void VMCI_ReleaseLock(VMCILock *lock, VMCILockFlags flags);
@@ -321,18 +304,20 @@ int VMCI_CopyFromUser(void *dst, VA64 src, size_t len);
 
 typedef void (VMCIWorkFn)(void *data);
 Bool VMCI_CanScheduleDelayedWork(void);
-int VMCI_ScheduleDelayedWork(VMCIWorkFn  *workFn,
-                             void *data);
+int VMCI_ScheduleDelayedWork(VMCIWorkFn *workFn, void *data);
 
 int VMCIMutex_Init(VMCIMutex *mutex);
 void VMCIMutex_Destroy(VMCIMutex *mutex);
 void VMCIMutex_Acquire(VMCIMutex *mutex);
 void VMCIMutex_Release(VMCIMutex *mutex);
 
-#if defined(SOLARIS) || defined(_WIN32)
+#if defined(SOLARIS) || defined(_WIN32) || defined(__APPLE__)
 int VMCIKernelIf_Init(void);
 void VMCIKernelIf_Exit(void);
-#endif // SOLARIS || _WIN32
+#if defined(_WIN32)
+void VMCIKernelIf_DrainDelayedWork(void);
+#endif // _WIN32
+#endif // SOLARIS || _WIN32 || __APPLE__
 
 #if !defined(VMKERNEL) && (defined(__linux__) || defined(_WIN32) || \
                            defined(SOLARIS) || defined(__APPLE__))
@@ -414,7 +399,7 @@ Bool VMCI_HostPersonalityActive(void);
 #  define VMCIList_InitEntry(_e)  List_InitElement(_e)
 #  define VMCIList_Empty(_l)   List_IsEmpty(_l)
 #  define VMCIList_Insert(_e, _l) List_Insert(_e, LIST_ATREAR(_l))
-#  define VMCIList_Remove(_e, _l) List_Remove(_e)
+#  define VMCIList_Remove(_e) List_Remove(_e)
 #  define VMCIList_Scan(_cur, _l) LIST_FORALL(_l, _cur)
 #  define VMCIList_ScanSafe(_cur, _next, _l) LIST_FORALL_SAFE(_l, _cur, _next)
 #  define VMCIList_Entry(_elem, _type, _field) List_Entry(_elem, _type, _field)
@@ -427,7 +412,7 @@ Bool VMCI_HostPersonalityActive(void);
 #  define VMCIList_InitEntry(_e)   DblLnkLst_Init(_e)
 #  define VMCIList_Empty(_l)   (!DblLnkLst_IsLinked(_l))
 #  define VMCIList_Insert(_e, _l) DblLnkLst_LinkLast(_l, _e)
-#  define VMCIList_Remove(_e, _l) DblLnkLst_Unlink1(_e)
+#  define VMCIList_Remove(_e) DblLnkLst_Unlink1(_e)
 #  define VMCIList_Scan(_cur, _l) DblLnkLst_ForEach(_cur, _l)
 #  define VMCIList_ScanSafe(_cur, _next, _l) DblLnkLst_ForEachSafe(_cur, _next, _l)
 #  define VMCIList_Entry(_elem, _type, _field) DblLnkLst_Container(_elem, _type, _field)
