@@ -226,13 +226,23 @@ VMCIContextGetDomainName(VMCIContext *context) // IN
 int
 VMCIContext_Init(void)
 {
-   VMCIList_Init(&contextList.head);
-   VMCI_InitLock(&contextList.lock, "VMCIContextListLock",
-                 VMCI_LOCK_RANK_HIGHER);
-   VMCI_InitLock(&contextList.firingLock, "VMCIContextFiringLock",
-                 VMCI_LOCK_RANK_MIDDLE_LOW);
+   int err;
 
-   return VMCI_SUCCESS;
+   VMCIList_Init(&contextList.head);
+
+   err = VMCI_InitLock(&contextList.lock, "VMCIContextListLock",
+                       VMCI_LOCK_RANK_HIGHER);
+   if (err < VMCI_SUCCESS) {
+      return err;
+   }
+
+   err = VMCI_InitLock(&contextList.firingLock, "VMCIContextFiringLock",
+                       VMCI_LOCK_RANK_MIDDLE_LOW);
+   if (err < VMCI_SUCCESS) {
+      VMCI_CleanupLock(&contextList.lock);
+   }
+
+   return err;
 }
 
 
@@ -340,9 +350,11 @@ VMCIContext_InitContext(VMCIId cid,                   // IN
       goto error;
    }
 
-   VMCI_InitLock(&context->lock,
-                 "VMCIContextLock",
-                 VMCI_LOCK_RANK_HIGHER);
+   result = VMCI_InitLock(&context->lock, "VMCIContextLock",
+                          VMCI_LOCK_RANK_HIGHER);
+   if (result < VMCI_SUCCESS) {
+      goto error;
+   }
    Atomic_Write(&context->refCount, 1);
 
    /* Inititialize host-specific VMCI context. */
@@ -443,7 +455,7 @@ VMCIContext_ReleaseContext(VMCIContext *context)   // IN
    /* Dequeue VMCI context. */
 
    VMCI_GrabLock(&contextList.lock, &flags);
-   VMCIList_Remove(&context->listItem, &contextList.head);
+   VMCIList_Remove(&context->listItem);
    VMCI_ReleaseLock(&contextList.lock, flags);
 
    VMCIContext_Release(context);
@@ -533,7 +545,7 @@ VMCIContextFreeContext(VMCIContext *context)  // IN
 
    VMCIList_ScanSafe(curr, next, &context->datagramQueue) {
       dqEntry = VMCIList_Entry(curr, DatagramQueueEntry, listItem);
-      VMCIList_Remove(curr, &context->datagramQueue);
+      VMCIList_Remove(curr);
       ASSERT(dqEntry && dqEntry->dg);
       ASSERT(dqEntry->dgSize == VMCI_DG_SIZE(dqEntry->dg));
       VMCI_FreeKernelMem(dqEntry->dg, dqEntry->dgSize);
@@ -598,27 +610,6 @@ VMCIContext_PendingDatagrams(VMCIId cid,      // IN
 }
 
 
-/*
- * We allow at least 1024 more event datagrams from the hypervisor past the
- * normally allowed datagrams pending for a given context.  We define this
- * limit on event datagrams from the hypervisor to guard against DoS attack
- * from a malicious VM which could repeatedly attach to and detach from a queue
- * pair, causing events to be queued at the destination VM.  However, the rate
- * at which such events can be generated is small since it requires a VM exit
- * and handling of queue pair attach/detach call at the hypervisor.  Event
- * datagrams may be queued up at the destination VM if it has interrupts
- * disabled or if it is not draining events for some other reason.  1024
- * datagrams is a grossly conservative estimate of the time for which
- * interrupts may be disabled in the destination VM, but at the same time does
- * not exacerbate the memory pressure problem on the host by much (size of each
- * event datagram is small).
- */
-
-#define VMCI_MAX_DATAGRAM_AND_EVENT_QUEUE_SIZE \
-   (VMCI_MAX_DATAGRAM_QUEUE_SIZE + \
-    1024 * (sizeof(VMCIDatagram) + sizeof(VMCIEventData_Max)))
-
-
 /*
  *----------------------------------------------------------------------
  *
@@ -923,7 +914,7 @@ VMCIContext_DequeueDatagram(VMCIContext *context, // IN
       return VMCI_ERROR_NO_MEM;
    }
 
-   VMCIList_Remove(listItem, &context->datagramQueue);
+   VMCIList_Remove(listItem);
    context->pendingDatagrams--;
    context->datagramQueueSize -= dqEntry->dgSize;
    if (context->pendingDatagrams == 0) {
