@@ -128,6 +128,27 @@ VMCIQPair_Alloc(VMCIQPair **qpair,            // OUT
    VMCIHandle dst = VMCI_MAKE_HANDLE(peer, VMCI_INVALID_ID);
    VMCIRoute route;
 
+   /*
+    * Restrict the size of a queuepair.  The device already enforces a limit
+    * on the total amount of memory that can be allocated to queuepairs for a
+    * guest.  However, we try to allocate this memory before we make the
+    * queuepair allocation hypercall.  On Windows and Mac OS, we request a
+    * single, continguous block, and it will fail if the OS cannot satisfy the
+    * request. On Linux, we allocate each page separately, which means rather
+    * than fail, the guest will thrash while it tries to allocate, and will
+    * become increasingly unresponsive to the point where it appears to be hung.
+    * So we place a limit on the size of an individual queuepair here, and
+    * leave the device to enforce the restriction on total queuepair memory.
+    * (Note that this doesn't prevent all cases; a user with only this much
+    * physical memory could still get into trouble.)  The error used by the
+    * device is NO_RESOURCES, so use that here too.
+    */
+
+   if (produceQSize + consumeQSize < MAX(produceQSize, consumeQSize) ||
+       produceQSize + consumeQSize > VMCI_MAX_GUEST_QP_MEMORY) {
+      return VMCI_ERROR_NO_RESOURCES;
+   }
+
    myQPair = VMCI_AllocKernelMem(sizeof *myQPair, VMCI_MEMORY_NONPAGED);
    if (!myQPair) {
       return VMCI_ERROR_NO_MEM;
@@ -209,22 +230,19 @@ VMCIQPair_Detach(VMCIQPair **qpair) // IN/OUT
    oldQPair = *qpair;
    result = VMCIQueuePair_Detach(oldQPair->handle, oldQPair->guestEndpoint);
 
-   if  (result >= VMCI_SUCCESS) {
-#ifdef DEBUG
-      oldQPair->handle = VMCI_INVALID_HANDLE;
-      oldQPair->produceQ = NULL;
-      oldQPair->consumeQ = NULL;
-      oldQPair->produceQSize = 0;
-      oldQPair->consumeQSize = 0;
-      oldQPair->flags = 0;
-      oldQPair->privFlags = 0;
-      oldQPair->peer = VMCI_INVALID_ID;
-#endif
+   /*
+    * The guest can fail to detach for a number of reasons, and if it does so,
+    * it will cleanup the entry (if there is one).  The host can fail too, but
+    * it won't cleanup the entry immediately, it will do that later when the
+    * context is freed.  Either way, we need to release the qpair struct here;
+    * there isn't much the caller can do, and we don't want to leak.
+    */
 
-      VMCI_FreeKernelMem(oldQPair, sizeof *oldQPair);
-
-      *qpair = NULL;
-   }
+   memset(oldQPair, 0, sizeof *oldQPair);
+   oldQPair->handle = VMCI_INVALID_HANDLE;
+   oldQPair->peer = VMCI_INVALID_ID;
+   VMCI_FreeKernelMem(oldQPair, sizeof *oldQPair);
+   *qpair = NULL;
 
    return result;
 }
