@@ -26,6 +26,7 @@
 
 #include <linux/file.h>
 #include <linux/fs.h>
+#include <linux/init.h>
 #if defined(__x86_64__) && LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 12)
 #   include <linux/ioctl32.h>
 /* Use weak: not all kernels export sys_ioctl for use by modules */
@@ -36,10 +37,8 @@ sys_ioctl(unsigned int fd, unsigned int cmd, unsigned long arg);
 #include <linux/moduleparam.h>
 #include <linux/poll.h>
 #include <linux/smp.h>
-#include <linux/smp_lock.h>
 
 #include "compat_highmem.h"
-#include "compat_init.h"
 #include "compat_interrupt.h"
 #include "compat_ioport.h"
 #include "compat_kernel.h"
@@ -490,10 +489,7 @@ LinuxDriver_Close(struct inode *inode, // IN
    ASSERT(vmciLinux);
 
    if (vmciLinux->ctType == VMCIOBJ_CONTEXT) {
-      VMCIId cid;
-
       ASSERT(vmciLinux->context);
-      cid = VMCIContext_GetId(vmciLinux->context);
 
       VMCIContext_ReleaseContext(vmciLinux->context);
       vmciLinux->context = NULL;
@@ -1862,7 +1858,10 @@ vmci_probe_device(struct pci_dev *pdev,           // IN: vmci PCI device
     * registered the handler (as the irq line may be shared).
     */
    VMCIUtil_Init();
-   VMCIQPGuestEndpoints_Init();
+
+   if (VMCIQPGuestEndpoints_Init() < VMCI_SUCCESS) {
+      goto util_exit;
+   }
 
    /*
     * Enable interrupts.  Try MSI-X first, then MSI, and then fallback on
@@ -1926,6 +1925,7 @@ vmci_probe_device(struct pci_dev *pdev,           // IN: vmci PCI device
 
  components_exit:
    VMCIQPGuestEndpoints_Exit();
+ util_exit:
    VMCIUtil_Exit();
    vmci_dev.enabled = FALSE;
    if (vmci_dev.intr_type == VMCI_INTR_TYPE_MSIX) {
