@@ -103,6 +103,11 @@ VSockAddr_InitNoFamily(struct sockaddr_vm *addr, // OUT
 {
    ASSERT(addr);
    memset(addr, 0, sizeof *addr);
+
+#if defined(__APPLE__)
+   addr->svm_len = sizeof *addr;
+#endif
+
    addr->svm_cid = cid;
    addr->svm_port = port;
    VSOCK_ADDR_NOFAMILY_ASSERT(addr);
@@ -441,7 +446,15 @@ VSockAddr_SocketContextDgram(uint32 cid,  // IN
                              uint32 rid)  // IN
 {
    if (cid == VMCI_HYPERVISOR_CONTEXT_ID) {
-      return FALSE;
+      /*
+       * Registrations of PBRPC Servers do not modify VMX/Hypervisor state and
+       * are allowed.
+       */
+      if (rid == VMCI_UNITY_PBRPC_REGISTER) {
+         return TRUE;
+      } else {
+         return FALSE;
+      }
    }
 
    return TRUE;
