@@ -42,7 +42,8 @@
 
 #define INCLUDE_ALLOW_USERLEVEL
 #define INCLUDE_ALLOW_VMKERNEL
-
+#define INCLUDE_ALLOW_DISTRIBUTE
+#define INCLUDE_ALLOW_VMKDRIVERS
 #define INCLUDE_ALLOW_VMCORE
 #define INCLUDE_ALLOW_MODULE
 #include "includeCheck.h"
