@@ -55,6 +55,7 @@
 #include "xdrutil.h"
 #include "vmsupport.h"
 #include "vmware/guestrpc/tclodefs.h"
+#include "vmware/tools/log.h"
 #include "vmware/tools/plugin.h"
 #include "vmware/tools/utils.h"
 #include "vmware/tools/vmbackup.h"
@@ -65,17 +66,6 @@
 VM_EMBED_VERSION(VMTOOLSD_VERSION_STRING);
 #endif
 
-
-/**
- * Upper bound for value sizes, based on gethostname(2):
- *
- *    SUS2 guarantees that "Host names are limited to 255 bytes".
- *
- * We can't modify MAX_VALUE_LEN since it's used in wire-level definitions
- * (such as PartitionEntry).
- */
-#define GUESTINFO_MAX_VALUE_SIZE    256
-
 MY_ASSERTS(GI_SIZE_ASSERTS,
            ASSERT_ON_COMPILE(GUESTINFO_MAX_VALUE_SIZE >= MAX_VALUE_LEN);
 )
@@ -1061,13 +1051,13 @@ TweakGatherLoop(ToolsAppCtx *ctx,
    guestInfoPollInterval = pollInterval;
 
    if (guestInfoPollInterval) {
-      g_message("New poll interval is %us.\n", guestInfoPollInterval / 1000);
+      g_info("New poll interval is %us.\n", guestInfoPollInterval / 1000);
 
       gatherTimeoutSource = g_timeout_source_new(guestInfoPollInterval);
       VMTOOLSAPP_ATTACH_SOURCE(ctx, gatherTimeoutSource, GuestInfoGather, ctx, NULL);
       g_source_unref(gatherTimeoutSource);
    } else {
-      g_message("Poll loop disabled.\n");
+      g_info("Poll loop disabled.\n");
    }
 
    g_clear_error(&gError);
