@@ -34,6 +34,10 @@ extern "C" {
 
 #include <X11/extensions/Xinerama.h>
 #include <X11/extensions/XTest.h>
+
+#ifndef NO_XCOMPOSITE
+#   include <X11/extensions/Xcomposite.h>
+#endif
 }
 
 typedef struct {
@@ -85,14 +89,7 @@ static Bool GetRelevantWMWindow(UnityPlatform *up,
 static Bool SetWindowStickiness(UnityPlatform *up,
                                 UnityWindowId windowId,
                                 Bool wantSticky);
-
-static const GuestCapabilities platformUnityCaps[] = {
-   UNITY_CAP_WORK_AREA,
-   UNITY_CAP_START_MENU,
-   UNITY_CAP_MULTI_MON,
-   UNITY_CAP_VIRTUAL_DESK,
-   UNITY_CAP_STICKY_WINDOWS,
-};
+static UnitySpecialWindow *MakeCompositeOverlaysObject(UnityPlatform *up);
 
 /*
  * Has to be global for UnityPlatformXErrorHandler
@@ -356,70 +353,6 @@ UnityPlatformCleanup(UnityPlatform *up) // IN
 }
 
 
-/*
- *----------------------------------------------------------------------------
- *
- * UnityPlatformRegisterCaps --
- *
- *      Register guest platform specific capabilities with the VMX.
- *
- * Results:
- *      None.
- *
- * Side effects:
- *      None.
- *
- *----------------------------------------------------------------------------
- */
-
-void
-UnityPlatformRegisterCaps(UnityPlatform *up) // IN
-{
-   ASSERT(up);
-
-   if (!RpcOut_sendOne(NULL, NULL, UNITY_RPC_SHOW_TASKBAR_CAP " %d",
-                       Unity_IsSupported() ? 1 : 0)) {
-      Debug("Could not set unity show taskbar cap\n");
-   }
-
-   AppUtil_SendGuestCaps(platformUnityCaps, ARRAYSIZE(platformUnityCaps), TRUE);
-}
-
-
-/*
- *----------------------------------------------------------------------------
- *
- * UnityPlatformUnregisterCaps --
- *
- *      Unregister guest platform specific capabilities with the VMX.
- *
- * Results:
- *      None.
- *
- * Side effects:
- *      None.
- *
- *----------------------------------------------------------------------------
- */
-
-void
-UnityPlatformUnregisterCaps(UnityPlatform *up) // IN
-{
-   /*
-    * This function may potentially be called during UnityPlatform destruction.
-    */
-   if (!up) {
-      return;
-   }
-
-   AppUtil_SendGuestCaps(platformUnityCaps, ARRAYSIZE(platformUnityCaps), FALSE);
-
-   if (!RpcOut_sendOne(NULL, NULL, UNITY_RPC_SHOW_TASKBAR_CAP " 0")) {
-      Debug("Failed to unregister Unity taskbar capability\n");
-   }
-}
-
-
 /*****************************************************************************
  * Unity main loop and event handling                                        *
  *****************************************************************************/
@@ -945,6 +878,7 @@ UnityPlatformEnterUnity(UnityPlatform *up) // IN
 
    XSync(up->display, TRUE);
    up->rootWindows = UnityPlatformMakeRootWindowsObject(up);
+   MakeCompositeOverlaysObject(up);
    up->isRunning = TRUE;
    up->eventTimeDiff = 0;
 
@@ -3340,6 +3274,66 @@ UnityPlatformSendMouseWheel(UnityPlatform *up,    // IN
 }
 
 
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * MakeCompositeOverlaysObject --
+ *
+ *      Probes X server for any composite overlay windows.  If found, they're
+ *      monitored as UnitySpecialWindows whereby we won't mistakenly place them
+ *      in the global window tracker.
+ *
+ * Results:
+ *      Returns pointer to new UnitySpecialWindow on success, NULL on failure.
+ *
+ * Side effects:
+ *      If overlay windows haven't yet been mapped, they will be (temporarily).
+ *
+ *      Caller doesn't need to free returned UnitySpecialWindow*.  That's handled
+ *      automatically when exiting Unity.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+static UnitySpecialWindow *
+MakeCompositeOverlaysObject(UnityPlatform *up)  // IN
+{
+   ASSERT(up);
+   ASSERT(up->rootWindows);
+
+#ifndef NO_XCOMPOSITE
+   int eventBase;
+   int errorBase;
+   if (XCompositeQueryExtension(up->display, &eventBase, &errorBase)) {
+      /*
+       * XCompositeGetOverlayWindow didn't appear until XComposite 0.3.
+       */
+      int major = 0;
+      int minor = 0;
+      XCompositeQueryVersion(up->display, &major, &minor);
+      if (major > 0 || (major == 0 && minor >= 3)) {
+         size_t nWindows = up->rootWindows->numWindows;
+         Window *overlays = (Window*)Util_SafeCalloc(nWindows, sizeof *overlays);
+         for (unsigned int i = 0; i < nWindows; i++) {
+            overlays[i] = XCompositeGetOverlayWindow(up->display,
+                                                     up->rootWindows->windows[i]);
+            XCompositeReleaseOverlayWindow(up->display, overlays[i]);
+         }
+
+         /*
+          * Note 1: See above. Caller doesn't need to need to track this explicitly.
+          * Note 2: The NULL event handler parameter is how we tell the X event handling
+          *         pieces to ignore this window, thereby keeping it out of the window
+          *         tracker.
+          */
+         return USWindowCreate(up, NULL, overlays, nWindows);
+      }
+   }
+#endif // ifndef NO_XCOMPOSITE
+
+   return NULL;
+}
+
 /*
  *
  * End file-scope functions.
